# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Default actions reside in this module.
These action are available out of the box for Murano users.

"""
import six
import yaql.language.utils as utils

import murano.dsl.exceptions as exceptions
import murano.dsl.murano_object as mo
import murano.policy.modify.actions.base as base


class ActionUtils(object):
    def _get_objects_by_id(self, obj, objects=None):
        if objects is None:
            objects = {}
        if isinstance(obj, mo.MuranoObject):
            objects[obj.object_id] = obj
            for prop_name in obj.type.properties:
                try:
                    prop_val = obj.get_property(prop_name)
                except exceptions.UninitializedPropertyAccessError:
                    continue
                self._get_objects_by_id(prop_val, objects)
        elif isinstance(obj, tuple):
            for item in obj:
                self._get_objects_by_id(item, objects)
        elif isinstance(obj, utils.FrozenDict):
            for k, v in six.iteritems(obj):
                self._get_objects_by_id(k, objects)
                self._get_objects_by_id(v, objects)
        return objects

    @staticmethod
    def _check_present(obj_id, objects):
        if obj_id not in objects.keys():
            raise ValueError('No such object, obj_id: {0}'
                             .format(obj_id))


class RemoveObjectAction(base.ModifyActionBase, ActionUtils):
    """Remove object from given model"""

    def __init__(self, object_id):
        """Initializes action parameters

        :param object_id: id of an object
        """
        self._object_id = object_id

    @staticmethod
    def _match_object_id(obj_id, obj):
        return isinstance(obj, mo.MuranoObject) and obj_id == obj.object_id

    def modify(self, obj):
        """Remove object from given model"""
        objects = self._get_objects_by_id(obj)
        self._check_present(self._object_id, objects)

        for _obj in objects.values():
            for prop_name in _obj.type.properties:
                try:
                    val = _obj.get_property(prop_name)
                except exceptions.UninitializedPropertyAccessError:
                    continue
                if self._match_object_id(self._object_id, val):
                    _obj.set_property(prop_name, None)
                # remove object from list
                elif isinstance(val, tuple):
                    filtered_list = list(val)
                    for item in val:
                        if self._match_object_id(self._object_id, item):
                            filtered_list.remove(item)
                    if len(filtered_list) < len(val):
                        _obj.set_property(prop_name, tuple(filtered_list))
                # remove object from dict
                elif isinstance(val, utils.FrozenDict):
                    filtered_dict = {k: v for k, v in val.items() if not
                                     self._match_object_id(self._object_id,
                                                           k) and not
                                     self._match_object_id(self._object_id, v)}
                    if len(filtered_dict) < len(val):
                        _obj.set_property(prop_name,
                                          utils.FrozenDict(filtered_dict))


class SetPropertyAction(base.ModifyActionBase, ActionUtils):
    """Set property on given object"""

    def __init__(self, object_id, prop_name, value):
        """Initializes action parameters

        :param object_id: id of an object
        :param prop_name: property name
        :param value: new value of the property
        """
        self._object_id = object_id
        self._prop_name = prop_name
        self._value = value

    def modify(self, obj):
        """Set property on given object in model"""
        objects = self._get_objects_by_id(obj)
        self._check_present(self._object_id, objects)
        target_obj = objects[self._object_id]
        target_obj.set_property(self._prop_name, self._value)


class RemoveRelationAction(SetPropertyAction):
    """Remove relation from given model"""

    def __init__(self, object_id, prop_name):
        super(RemoveRelationAction, self).__init__(object_id, prop_name, None)


class AddObjectAction(base.ModifyActionBase, ActionUtils):
    """Add new object to object model"""

    def __init__(self, owner_id, owner_relation, type, init_args):
        """Initializes action parameters

        :param owner_id: id of an owner
        :param owner_relation: name of relation on owner
        :param type: new object type
        :param init_args: properties of the new object
        """
        self._init_args = init_args
        self._type = type
        self._owner_relation = owner_relation
        self._owner_id = owner_id

    def modify(self, model):
        """Creates new object and adds it to the model"""
        new_obj = {'type': type}
        new_obj.update(self._init_args)
        objects = self._get_objects_by_id(model)
        self._check_present(self._owner_id, objects)
        owner = objects[self._owner_id]
        # adding objects to list or dict is not supported for now
        owner.set_property(self._owner_relation, new_obj)


class AddRelationAction(base.ModifyActionBase, ActionUtils):
    """Adds relation between two existing objects within model"""

    def __init__(self, source_id, relation, target_id):
        """Initializes action parameters

        :param source_id: id of an relation source
        :param relation: name of relation on source object
        :param target_id: id of an relation target
        """
        self._source_id = source_id
        self._relation = relation
        self._target_id = target_id

    def modify(self, model):
        """Creates new object and adds it to the model"""
        objects = self._get_objects_by_id(model)
        self._check_present(self._source_id, objects)
        self._check_present(self._target_id, objects)
        source = objects[self._source_id]
        target = objects[self._target_id]
        # adding objects to list or dict is not supported for now
        source.set_property(self._relation, target)



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import inspect
import os.path

import six
from yaql import specs

from murano.dsl import constants
from murano.dsl import dsl
from murano.dsl import dsl_types
from murano.dsl import helpers
from murano.dsl import yaql_integration


@dsl.name('io.murano.StackTrace')
class StackTrace(object):
    def __init__(self, this, context, include_native_frames=True):
        frames = []
        caller_context = context
        while True:
            if not caller_context:
                break
            frame = compose_stack_frame(caller_context)
            frames.append(frame)
            caller_context = helpers.get_caller_context(caller_context)
        frames.reverse()
        frames.pop()

        if include_native_frames:
            native_frames = []
            for frame in inspect.trace()[1:]:
                location = dsl_types.ExpressionFilePosition(
                    os.path.abspath(frame[1]), frame[2],
                    -1, frame[2], -1)
                method = frame[3]
                native_frames.append({
                    'instruction': frame[4][0].strip(),
                    'location': location,
                    'methodName': method,
                    'typeName': None
                })
            frames.extend(native_frames)

        this.properties.frames = frames

    @specs.meta(constants.META_NO_TRACE, True)
    @specs.meta('Usage', 'Action')
    def to_string(self, this, prefix=''):
        return '\n'.join([format_frame(t, prefix)for t in this['frames']])


def compose_stack_frame(context):
    instruction = helpers.get_current_instruction(context)
    method = helpers.get_current_method(context)
    return {
        'instruction': None if instruction is None
        else six.text_type(instruction),

        'location': None if instruction is None
        else instruction.source_file_position,

        'methodName': None if method is None else method.name,
        'typeName': None if method is None else method.declaring_type.name
    }


def format_frame(frame, prefix=''):
    instruction = frame['instruction']
    method_name = frame['methodName']
    type_name = frame['typeName']
    location = frame['location']
    if type_name:
        method_name += ' of type ' + type_name

    if location:
        args = (
            os.path.abspath(location.file_path),
            location.start_line,
            ':' + str(location.start_column)
            if location.start_column >= 0 else '',
            method_name,
            instruction,
            prefix
        )
        return (u'{5}File "{0}", line {1}{2} in method {3}\n'
                u'{5}    {4}').format(*args)
    else:
        return u'{2}File <unknown> in method {0}\n{2}    {1}'.format(
            method_name, instruction, prefix)


def create_stack_trace(context, include_native_frames=True):
        stacktrace = yaql_integration.call_func(
            context, 'new', 'io.murano.StackTrace',
            includeNativeFrames=include_native_frames)
        return dsl.MuranoObjectInterface.create(stacktrace)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
import sqlalchemy as sa
from sqlalchemy.dialects import mysql


def LargeBinary():
    return sa.LargeBinary().with_variant(mysql.LONGBLOB(), 'mysql')


class JsonBlob(sa.TypeDecorator):
    impl = sa.Text

    def process_bind_param(self, value, dialect):
        return jsonutils.dumps(value)

    def process_result_value(self, value, dialect):
        if value is not None:
            return jsonutils.loads(value)
        return None



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json as jsonlib

import yaml as yamllib
from yaql.language import specs
from yaql.language import yaqltypes

from murano.dsl import dsl
from murano.dsl import dsl_types
from murano.dsl import helpers

if hasattr(yamllib, 'CSafeLoader'):
    yaml_loader = yamllib.CSafeLoader
else:
    yaml_loader = yamllib.SafeLoader


def _construct_yaml_str(self, node):
    # Override the default string handling function
    # to always return unicode objects
    return self.construct_scalar(node)

yaml_loader.add_constructor(u'tag:yaml.org,2002:str', _construct_yaml_str)
# Unquoted dates like 2013-05-23 in yaml files get loaded as objects of type
# datetime.data which causes problems in API layer when being processed by
# oslo.serialization.jsonutils. Therefore, make unicode string out of
# timestamps until jsonutils can handle dates.
yaml_loader.add_constructor(u'tag:yaml.org,2002:timestamp',
                            _construct_yaml_str)


@dsl.name('io.murano.system.Resources')
class ResourceManager(object):
    def __init__(self, context):
        murano_class = helpers.get_type(helpers.get_caller_context(context))
        self._package = murano_class.package

    @staticmethod
    @specs.parameter('owner', dsl.MuranoTypeParameter(nullable=True))
    @specs.inject('receiver', yaqltypes.Receiver())
    def string(receiver, name, owner=None, binary=False):
        path = ResourceManager._get_package(owner, receiver).get_resource(name)
        mode = 'rb' if binary else 'rU'
        with open(path, mode) as file:
            return file.read()

    @classmethod
    @specs.parameter('owner', dsl.MuranoTypeParameter(nullable=True))
    @specs.inject('receiver', yaqltypes.Receiver())
    def json(cls, receiver, name, owner=None):
        return jsonlib.loads(cls.string(receiver, name, owner))

    @classmethod
    @specs.parameter('owner', dsl.MuranoTypeParameter(nullable=True))
    @specs.inject('receiver', yaqltypes.Receiver())
    def yaml(cls, receiver, name, owner=None):
        return yamllib.load(
            cls.string(receiver, name, owner), Loader=yaml_loader)

    @staticmethod
    def _get_package(owner, receiver):
        if owner is None:
            if isinstance(receiver, dsl_types.MuranoObjectInterface):
                return receiver.extension._package
            murano_class = helpers.get_type(helpers.get_caller_context())
        else:
            murano_class = owner.type
        return murano_class.package



# Copyright (c) 2015 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import credentials_factory as common_creds
from tempest import config
from tempest.lib import auth

from murano_tempest_tests.services.application_catalog \
    import application_catalog_client
from murano_tempest_tests.services.artifacts import artifacts_client
from murano_tempest_tests.services.service_broker import service_broker_client

CONF = config.CONF


class Manager(object):
    def __init__(self,
                 credentials=common_creds.get_configured_admin_credentials(
                     'identity_admin')):
        self.auth_provider = get_auth_provider(credentials)
        self.service_broker_client = service_broker_client.ServiceBrokerClient(
            self.auth_provider)
        if CONF.application_catalog.glare_backend:
            self.artifacts_client = artifacts_client.ArtifactsClient(
                self.auth_provider)
        else:
            self.artifacts_client = None
        self.application_catalog_client = \
            application_catalog_client.ApplicationCatalogClient(
                self.auth_provider)


class AltManager(Manager):
    def __init__(self, service=None):
        super(AltManager, self).__init__(
            common_creds.get_configured_admin_credentials('alt_user'), service)


def get_auth_provider(credentials, scope='project'):
    default_params = {
        'disable_ssl_certificate_validation':
            CONF.identity.disable_ssl_certificate_validation,
        'ca_certs': CONF.identity.ca_certificates_file,
        'trace_requests': CONF.debug.trace_requests
    }

    if isinstance(credentials, auth.KeystoneV3Credentials):
        auth_provider_class, auth_url = \
            auth.KeystoneV3AuthProvider, CONF.identity.uri_v3
    else:
        auth_provider_class, auth_url = \
            auth.KeystoneV2AuthProvider, CONF.identity.uri

    _auth_provider = auth_provider_class(credentials, auth_url,
                                         scope=scope,
                                         **default_params)
    _auth_provider.set_auth()
    return _auth_provider



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import mock
import six
from yaql import contexts
from yaql import specs

from murano.dsl import constants
from murano.dsl import executor
from murano.dsl import murano_type
from murano.engine import execution_session
from murano.engine import mock_context_manager
from murano.engine.system import test_fixture
from murano.tests.unit import base
from murano.tests.unit.dsl.foundation import object_model as om
from murano.tests.unit.dsl.foundation import runner
from murano.tests.unit.dsl.foundation import test_case


FIXTURE_CLASS = 'io.murano.system.Agent'
FIXTURE_FUNC = 'call'


def _get_fd(set_to_extract):
    return list(set_to_extract)[0]


class TestMockContextManager(mock_context_manager.MockContextManager):
    def __init__(self, functions):
        super(TestMockContextManager, self).__init__()
        self.__functions = functions

    def create_root_context(self, runtime_version):
        root_context = super(TestMockContextManager, self).create_root_context(
            runtime_version)
        context = root_context.create_child_context()
        for name, func in six.iteritems(self.__functions):
            context.register_function(func, name)
        return context


class MockRunner(runner.Runner):
    def __init__(self, model, package_loader, functions):
        if isinstance(model, six.string_types):
            model = om.Object(model)
        model = om.build_model(model)
        if 'Objects' not in model:
            model = {'Objects': model}
        self.executor = executor.MuranoDslExecutor(
            package_loader, TestMockContextManager(functions),
            execution_session.ExecutionSession())
        self._root = self.executor.load(model).object


class TestMockManager(base.MuranoTestCase):

    def test_create_type_context(self):
        mock_manager = mock_context_manager.MockContextManager()
        mock_murano_class = mock.MagicMock(spec=murano_type.MuranoClass)
        mock_murano_class.name = FIXTURE_CLASS
        original_function = mock.MagicMock(spec=specs.FunctionDefinition)
        original_function.is_method = True
        original_function.name = FIXTURE_FUNC
        original_context = contexts.Context()
        p = mock.patch("inspect.getargspec", new=mock.MagicMock())
        p.start()
        original_context.register_function(original_function)
        mock_murano_class.context = original_context
        p.stop()

        mock_function = mock.MagicMock(spec=specs.FunctionDefinition)
        mock_function.is_method = True
        mock_function.name = FIXTURE_FUNC

        mock_manager.class_mock_ctx[FIXTURE_CLASS] = [mock_function]

        result_context = mock_manager.create_type_context(mock_murano_class)
        all_functions = result_context.collect_functions(FIXTURE_FUNC)

        # Mock function should go first, but result context should contain both
        self.assertIs(mock_function, _get_fd(all_functions[0]))
        self.assertIs(original_function, _get_fd(all_functions[1]))

    def test_create_root_context(self):
        mock_manager = mock_context_manager.MockContextManager()
        ctx_to_check = mock_manager.create_root_context(
            constants.RUNTIME_VERSION_1_1)
        inject_count = ctx_to_check.collect_functions('inject')
        with_original_count = ctx_to_check.collect_functions('withOriginal')

        self.assertEqual(2, len(inject_count[0]))
        self.assertEqual(1, len(with_original_count[0]))


class TestMockYaqlFunctions(test_case.DslTestCase):

    def setUp(self):
        super(TestMockYaqlFunctions, self).setUp()
        self.package_loader.load_package('io.murano', None).register_class(
            test_fixture.TestFixture)
        self.runner = MockRunner(om.Object('TestMocks'),
                                 self.package_loader, self._functions)

    def test_inject_method_with_str(self):
        self.runner.testInjectMethodWithString()

    def test_inject_object_with_str(self):
        self.runner.testInjectObjectWithString()

    def test_inject_method_with_yaql_expr(self):
        self.runner.testInjectMethodWithYaqlExpr()

    def test_inject_method_with_yaql_expr2(self):
        self.runner.testInjectMethodWithYaqlExpr2()

    def test_inject_object_with_yaql_expr(self):
        self.runner.testInjectObjectWithYaqlExpr()

    def test_with_original(self):
        self.runner.testWithoriginal()

    def test_original_method(self):
        self.runner.testOriginalMethod()



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import semantic_version


EXPRESSION_MEMORY_QUOTA = 512 * 1024
ITERATORS_LIMIT = 2000

CTX_ACTIONS_ONLY = '?actionsOnly'
CTX_ALLOW_PROPERTY_WRITES = '$?allowPropertyWrites'
CTX_ARGUMENT_OWNER = '$?argumentOwner'
CTX_CALLER_CONTEXT = '$?callerContext'
CTX_CURRENT_INSTRUCTION = '$?currentInstruction'
CTX_CURRENT_EXCEPTION = '$?currentException'
CTX_CURRENT_METHOD = '$?currentMethod'
CTX_NAMES_SCOPE = '$?namesScope'
CTX_ORIGINAL_CONTEXT = '$?originalContext'
CTX_SKIP_FRAME = '$?skipFrame'
CTX_THIS = '$?this'
CTX_TYPE = '$?type'
CTX_VARIABLE_SCOPE = '$?variableScope'
CTX_YAQL_ENGINE = '$?yaqlEngine'

DM_OBJECTS = 'Objects'
DM_OBJECTS_COPY = 'ObjectsCopy'
DM_ATTRIBUTES = 'Attributes'

META_MURANO_METHOD = '?muranoMethod'
META_NO_TRACE = '?noTrace'
META_MPL_META = 'Meta'
META_USAGE = 'Usage'
META_SCOPE = 'Scope'

CORE_LIBRARY = 'io.murano'
CORE_LIBRARY_OBJECT = 'io.murano.Object'

TL_CONTEXT = '__murano_context'
TL_ID = '__thread_id'
TL_OBJECT_STORE = '__murano_object_store'
TL_SESSION = '__murano_execution_session'
TL_CONTRACT_PASSKEY = '__murano_contract_passkey'
TL_OBJECTS_DRY_RUN = '__murano_objects_dry_run'


RUNTIME_VERSION_1_0 = semantic_version.Version('1.0.0')
RUNTIME_VERSION_1_1 = semantic_version.Version('1.1.0')
RUNTIME_VERSION_1_2 = semantic_version.Version('1.2.0')
RUNTIME_VERSION_1_3 = semantic_version.Version('1.3.0')
RUNTIME_VERSION_1_4 = semantic_version.Version('1.4.0')



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools as func

from oslo_log import log as logging
from webob import exc

from murano.api.v1 import request_statistics
from murano.common.helpers import token_sanitizer
from murano.common.i18n import _
from murano.common import wsgi
from murano.db.services import core_services
from murano import utils


LOG = logging.getLogger(__name__)

API_NAME = 'Services'


def normalize_path(f):
    @func.wraps(f)
    def f_normalize_path(*args, **kwargs):
        if 'path' in kwargs:
            if kwargs['path']:
                kwargs['path'] = '/services/' + kwargs['path']
            else:
                kwargs['path'] = '/services'
        return f(*args, **kwargs)

    return f_normalize_path


class Controller(object):
    @request_statistics.stats_count(API_NAME, 'Index')
    @utils.verify_env
    @normalize_path
    def get(self, request, environment_id, path):
        LOG.debug('Services:Get <EnvId: {env_id}, '
                  'Path: {path}>'.format(env_id=environment_id, path=path))

        session_id = None
        if hasattr(request, 'context') and request.context.session:
            session_id = request.context.session

        try:
            result = core_services.CoreServices.get_data(environment_id,
                                                         path,
                                                         session_id)
        except (KeyError, ValueError, AttributeError):
            raise exc.HTTPNotFound
        return result

    @request_statistics.stats_count(API_NAME, 'Create')
    @utils.verify_session
    @utils.verify_env
    @normalize_path
    def post(self, request, environment_id, path, body=None):
        if not body:
            msg = _('Request body is empty: please, provide '
                    'application object model')
            LOG.error(msg)
            raise exc.HTTPBadRequest(msg)
        secure_data = token_sanitizer.TokenSanitizer().sanitize(body)
        LOG.debug('Services:Post <EnvId: {env_id}, Path: {path}, '
                  'Body: {body}>'.format(env_id=environment_id,
                                         body=secure_data, path=path))

        post_data = core_services.CoreServices.post_data
        session_id = request.context.session
        try:
            result = post_data(environment_id, session_id, body, path)
        except (KeyError, ValueError):
            raise exc.HTTPNotFound
        return result

    @request_statistics.stats_count(API_NAME, 'Update')
    @utils.verify_session
    @utils.verify_env
    @normalize_path
    def put(self, request, environment_id, path, body=None):
        if not body:
            msg = _('Request body is empty: please, provide '
                    'application object model')
            LOG.error(msg)
            raise exc.HTTPBadRequest(msg)
        LOG.debug('Services:Put <EnvId: {0}, Path: {2}, '
                  'Body: {1}>'.format(environment_id, body, path))

        put_data = core_services.CoreServices.put_data
        session_id = request.context.session

        try:
            result = put_data(environment_id, session_id, body, path)
        except (KeyError, ValueError):
            raise exc.HTTPNotFound
        return result

    @request_statistics.stats_count(API_NAME, 'Delete')
    @utils.verify_session
    @utils.verify_env
    @normalize_path
    def delete(self, request, environment_id, path):
        LOG.debug('Services:Delete <EnvId: {0}, '
                  'Path: {1}>'.format(environment_id, path))

        delete_data = core_services.CoreServices.delete_data
        session_id = request.context.session

        try:
            delete_data(environment_id, session_id, path)
        except (KeyError, ValueError):
            raise exc.HTTPNotFound


def create_resource():
    return wsgi.Resource(Controller())



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
import oslo_messaging as messaging

from murano.common import uuidutils
from murano.dsl import dsl

CONF = cfg.CONF

UNCLASSIFIED = 0
APPLICATION = 100
OS_INSTANCE = 200


@dsl.name('io.murano.system.InstanceNotifier')
class InstanceReportNotifier(object):
    transport = None

    def __init__(self, environment):
        if InstanceReportNotifier.transport is None:
            InstanceReportNotifier.transport = messaging.get_transport(CONF)
        self._notifier = messaging.Notifier(
            InstanceReportNotifier.transport,
            publisher_id=uuidutils.generate_uuid(),
            topic='murano')
        self._environment_id = environment.id

    def _track_instance(self, instance, instance_type,
                        type_title, unit_count):
        payload = {
            'instance': instance.id,
            'environment': self._environment_id,
            'instance_type': instance_type,
            'type_name': instance.type.name,
            'type_title': type_title,
            'unit_count': unit_count
        }

        self._notifier.info({}, 'murano.track_instance', payload)

    def _untrack_instance(self, instance, instance_type):
        payload = {
            'instance': instance.id,
            'environment': self._environment_id,
            'instance_type': instance_type,
        }

        self._notifier.info({}, 'murano.untrack_instance', payload)

    def track_application(self, instance, title=None, unit_count=None):
        self._track_instance(instance, APPLICATION, title, unit_count)

    def untrack_application(self, instance):
        self._untrack_instance(instance, APPLICATION)

    def track_cloud_instance(self, instance):
        self._track_instance(instance, OS_INSTANCE, None, 1)

    def untrack_cloud_instance(self, instance):
        self._untrack_instance(instance, OS_INSTANCE)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_middleware import request_id as oslo_request_id
from oslo_serialization import jsonutils

from murano.common.i18n import _
from murano.common import wsgi
import murano.context

context_opts = [
    cfg.StrOpt('admin_role', default='admin',
               help=_('Role used to identify an authenticated user as '
                      'administrator.'))]

CONF = cfg.CONF
CONF.register_opts(context_opts)
CONF = cfg.CONF


class ContextMiddleware(wsgi.Middleware):
    def process_request(self, req):

        """Convert authentication information into a request context

        Generate a murano.context.RequestContext object from the available
        authentication headers and store on the 'context' attribute
        of the req object.

        :param req: wsgi request object that will be given the context object
        """
        roles = [r.strip() for r in req.headers.get('X-Roles').split(',')]
        kwargs = {
            'user': req.headers.get('X-User-Id'),
            'tenant': req.headers.get('X-Tenant-Id'),
            'auth_token': req.headers.get('X-Auth-Token'),
            'session': req.headers.get('X-Configuration-Session'),
            'is_admin': CONF.admin_role in roles,
            'request_id': req.environ.get(oslo_request_id.ENV_REQUEST_ID),
            'roles': roles
        }
        sc_header = req.headers.get('X-Service-Catalog')
        if sc_header:
            kwargs['service_catalog'] = jsonutils.loads(sc_header)
        req.context = murano.context.RequestContext(**kwargs)

    @classmethod
    def factory(cls, global_conf, **local_conf):
        def filter(app):
            return cls(app)
        return filter



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import functools as func
import re

import jsonschema
from oslo_log import log as logging
import six

from murano.common.i18n import _


LOG = logging.getLogger(__name__)


class TraverseHelper(object):
    value_type = (six.string_types, int, float, bool)

    @staticmethod
    def get(path, source):
        """Provides the ability to traverse a data source

        Provides the ability to traverse a data source made up of any
        combination of lists and dicts. Has simple rules for selecting item of
        the list:

        * each item should have id property
        * to select item from the list, specify id value

        Examples:
            source = {'obj': {'attr': True}}
            value = TraverseHelper.get('/obj/attr', source)

            source = {'obj': [
                {'id': '1', 'value': 1},
                {'id': '2s', 'value': 2},
            ]}
            value = TraverseHelper.get('/obj/2s/value', source)


        :param path: string with path to desired value
        :param source: python object (list or dict)
        :return: object
        :raise: ValueError if object is malformed
        """
        queue = collections.deque(filter(lambda x: x, path.split('/')))

        while len(queue):
            path = queue.popleft()

            if isinstance(source, list):
                idx_source = source
                iterator = (
                    i for i in source
                    if i.get('?', {}).get('id') == path
                )
                source = next(iterator, None)
                if source is None and path.isdigit():
                    source = idx_source[int(path)]
            elif isinstance(source, dict):
                source = source[path]
            elif isinstance(source, TraverseHelper.value_type):
                break
            else:
                raise ValueError(_('Source object or path is malformed'))

        return source

    @staticmethod
    def update(path, value, source):
        """Updates value selected with specified path.

        Warning: Root object could not be updated

        :param path: string with path to desired value
        :param value: value
        :param source: python object (list or dict)
        """
        parent_path = '/'.join(path.split('/')[:-1])
        node = TraverseHelper.get(parent_path, source)
        key = path[1:].split('/')[-1]
        if is_number(key):
            node[int(key)] = value
        else:
            node[key] = value

    @staticmethod
    def insert(path, value, source):
        """Inserts new item to selected list.

        :param path: string with path to desired value
        :param value: value
        :param source: List
        """
        node = TraverseHelper.get(path, source)
        node.append(value)

    @staticmethod
    def extend(path, value, source):
        """Extend list by appending elements from the iterable.

        :param path: string with path to desired value
        :param value: value
        :param source: List
        """
        node = TraverseHelper.get(path, source)
        node.extend(value)

    @staticmethod
    def remove(path, source):
        """Removes selected item from source.

        :param path: string with path to desired value
        :param source: python object (list or dict)
        """
        parent_path = '/'.join(path.split('/')[:-1])
        node = TraverseHelper.get(parent_path, source)
        key = path[1:].split('/')[-1]

        if isinstance(node, list):
            iterator = (i for i in node if i.get('?', {}).get('id') == key)
            item = next(iterator, None)
            if item is None and key.isdigit():
                del node[int(key)]
            else:
                node.remove(item)
        elif isinstance(node, dict):
            del node[key]
        else:
            raise ValueError(_('Source object or path is malformed'))


def is_number(var):
    try:
        int(var)
        return True
    except Exception:
        return False


def is_different(obj1, obj2):
    """Stripped-down version of deep.diff comparator

    Compares arbitrary nested objects, handles circular links, but doesn't
    point to the first difference as deep.diff does.
    """
    class Difference(Exception):
        pass

    def is_in(o, st):
        for _o in st:
            if o is _o:
                return True
        return False

    def rec(o1, o2, stack1=(), stack2=()):
        if is_in(o1, stack1) and is_in(o2, stack2):
            # circular reference detected - break the loop
            return
        elif is_in(o1, stack1):
            raise Difference()
        else:
            stack1 += (o1,)
            stack2 += (o2,)

        if o1 is o2:
            return
        elif (isinstance(o1, six.string_types) and
                isinstance(o2, six.string_types)) and o1 == o2:
            return
        elif type(o1) != type(o2):
            raise Difference()
        elif isinstance(o1, dict):
            # check for keys inequality
            rec(o1.keys(), o2.keys(), stack1, stack2)
            for key in o1.keys():
                rec(o1[key], o2[key], stack1, stack2)
        elif isinstance(o1, (list, tuple, set)):
            if len(o1) != len(o2):
                raise Difference()
            else:
                for _o1, _o2 in zip(o1, o2):
                    rec(_o1, _o2, stack1, stack2)
        elif hasattr(o1, '__dict__'):
            return rec(o1.__dict__, o2.__dict__, stack1, stack2)
        elif o1 != o2:
            raise Difference()

    try:
        rec(obj1, obj2)
    except Difference:
        return True
    else:
        return False


def build_entity_map(value):
    def build_entity_map_recursive(value, id_map):
        if isinstance(value, dict):
            if '?' in value and 'id' in value['?']:
                id_map[value['?']['id']] = value
            for v in six.itervalues(value):
                build_entity_map_recursive(v, id_map)
        if isinstance(value, list):
            for item in value:
                build_entity_map_recursive(item, id_map)

    id_map = {}
    build_entity_map_recursive(value, id_map)
    return id_map


def handle(f):
    """Handles exception in wrapped function and writes to LOG."""

    @func.wraps(f)
    def f_handle(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except Exception as e:
            LOG.exception(e)

    return f_handle


def validate_body(schema):
    def deco_validate_body(f):
        @func.wraps(f)
        def f_validate_body(*args, **kwargs):
            if 'body' in kwargs:
                jsonschema.validate(kwargs['body'], schema)
                return f(*args, **kwargs)
        return f_validate_body
    return deco_validate_body


def validate_quotes(value):
    """Validate filter values

    Validation opening/closing quotes in the expression.
    """
    open_quotes = True
    count_backslash_in_row = 0
    for i in range(len(value)):
        if value[i] == '"':
            if count_backslash_in_row % 2:
                continue
            if open_quotes:
                if i and value[i - 1] != ',':
                    msg = _("Invalid filter value %s. There is no comma "
                            "before opening quotation mark.") % value
                    raise ValueError(msg)
            else:
                if i + 1 != len(value) and value[i + 1] != ",":
                    msg = _("Invalid filter value %s. There is no comma "
                            "after opening quotation mark.") % value
                    raise ValueError(msg)
            open_quotes = not open_quotes
        elif value[i] == '\\':
            count_backslash_in_row += 1
        else:
            count_backslash_in_row = 0
    if not open_quotes:
        msg = _("Invalid filter value %s. The quote is not closed.") % value
        raise ValueError(msg)
    return True


def split_for_quotes(value):
    """Split filter values

    Split values by commas and quotes for 'in' operator, according api-wg.
    """
    validate_quotes(value)
    tmp = re.compile(r'''
        "(                 # if found a double-quote
           [^\"\\]*        # take characters either non-quotes or backslashes
           (?:\\.          # take backslashes and character after it
            [^\"\\]*)*     # take characters either non-quotes or backslashes
         )                 # before double-quote
        ",?                # a double-quote with comma maybe
        | ([^,]+),?        # if not found double-quote take any non-comma
                           # characters with comma maybe
        | ,                # if we have only comma take empty string
        ''', re.VERBOSE)
    val_split = [val[0] or val[1] for val in re.findall(tmp, value)]
    replaced_inner_quotes = [s.replace(r'\"', '"') for s in val_split]
    return replaced_inner_quotes



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_utils import timeutils
from webob import exc

from murano.common.i18n import _
from murano.common import utils
from murano.db.services import environment_templates as env_temp
from murano.db.services import environments as envs

LOG = logging.getLogger(__name__)


class CoreServices(object):
    @staticmethod
    def get_service_status(environment_id, service_id):
        """Service can have one of three distinguished statuses:

         - Deploying: if environment has status deploying and there is at least
            one orchestration engine report for this service;
         - Pending: if environment has status `deploying` and there is no
            report from orchestration engine about this service;
         - Ready: If environment has status ready.

        :param environment_id: Service environment, we always know to which
            environment service belongs to
        :param service_id: Id of service for which we checking status.
        :return: Service status
        """
        # Now we assume that service has same status as environment.
        # TODO(ruhe): implement as designed and described above

        return envs.EnvironmentServices.get_status(environment_id)

    @staticmethod
    def get_data(environment_id, path, session_id=None):
        get_description = envs.EnvironmentServices.get_environment_description

        env_description = get_description(environment_id, session_id)

        if env_description is None:
            return None

        if 'services' not in env_description:
            return []

        result = utils.TraverseHelper.get(path, env_description)

        if path == '/services':
            get_status = CoreServices.get_service_status
            for srv in result:
                srv['?']['status'] = get_status(environment_id, srv['?']['id'])

        return result

    @staticmethod
    def get_template_data(env_template_id, path):
        """It obtains the data for the template.

        It includes all the services. In case the path includes information
        such as the env_template_id, the information provided will
        be related to the entity specified in the path

        :param env_template_id: The env_template_id to obtain the data
        :param path: Id of service for which we checking status.
        :return: The template description
        """
        temp_description = env_temp.EnvTemplateServices.\
            get_description(env_template_id)

        if temp_description is None:
            return None

        if 'services' not in temp_description:
            return []

        result = utils.TraverseHelper.get(path, temp_description)
        if result is None:
            msg = _('Environment Template <EnvId {id}> is not found').format(
                id=env_template_id)
            LOG.error(msg)
            raise exc.HTTPNotFound(explanation=msg)
        return result

    @staticmethod
    def post_env_template_data(env_template_id, data, path):
        """It stores the template data inside the template description.

        :param env_template_id: The env_template_id to obtain the data
        :param data: the template description
        :param path: Id of service for which we checking status.
        :return: The template description
        """
        get_description = env_temp.EnvTemplateServices.get_description
        save_description = env_temp.EnvTemplateServices.save_description

        temp_description = get_description(env_template_id)
        if temp_description is None:
            msg = _('Environment Template <EnvId {id}> is not found').format(
                id=env_template_id)
            LOG.error(msg)
            raise exc.HTTPNotFound(explanation=msg)

        if 'services' not in temp_description:
            temp_description['services'] = []

        if path == '/services':
            if isinstance(data, list):
                utils.TraverseHelper.extend(path, data, temp_description)
            else:
                utils.TraverseHelper.insert(path, data, temp_description)
        save_description(temp_description)
        return data

    @staticmethod
    def post_application_data(env_template_id, data, path):
        """It stores the application data inside the template description.

        :param env_template_id: The env_template_id to obtain the data
        :param data: the template description
        :param path: Id of service for which we checking status.
        :return: The template description
        """
        get_description = env_temp.EnvTemplateServices.get_description
        save_description = env_temp.EnvTemplateServices.save_description

        temp_description = get_description(env_template_id)
        if temp_description is None:
            msg = _('Environment Template <EnvId {0}> is not found').format(
                env_template_id)
            LOG.error(msg)
            raise exc.HTTPNotFound(explanation=msg)

        if 'services' not in temp_description:
            temp_description['services'] = []

        if path == '/services':
            if isinstance(data, list):
                utils.TraverseHelper.extend(path, data, temp_description)
            else:
                utils.TraverseHelper.insert(path, data, temp_description)

        save_description(temp_description, env_template_id)

        return data

    @staticmethod
    def post_data(environment_id, session_id, data, path):
        get_description = envs.EnvironmentServices.get_environment_description
        save_description = envs.EnvironmentServices.\
            save_environment_description

        env_description = get_description(environment_id, session_id)
        if env_description is None:
            msg = _('Environment <EnvId {0}> is not found').format(
                environment_id)
            LOG.error(msg)
            raise exc.HTTPNotFound(explanation=msg)

        if 'services' not in env_description:
            env_description['services'] = []

        if path == '/services':
            if isinstance(data, list):
                utils.TraverseHelper.extend(path, data, env_description)
            else:
                utils.TraverseHelper.insert(path, data, env_description)

        save_description(session_id, env_description)

        return data

    @staticmethod
    def put_data(environment_id, session_id, data, path):
        get_description = envs.EnvironmentServices.get_environment_description
        save_description = envs.EnvironmentServices.\
            save_environment_description

        env_description = get_description(environment_id, session_id)

        utils.TraverseHelper.update(path, data, env_description)
        env_description['?']['updated'] = str(timeutils.utcnow())

        save_description(session_id, env_description)

        return data

    @staticmethod
    def delete_data(environment_id, session_id, path):
        get_description = envs.EnvironmentServices.get_environment_description
        save_description = envs.EnvironmentServices.\
            save_environment_description

        env_description = get_description(environment_id, session_id)

        utils.TraverseHelper.remove(path, env_description)
        save_description(session_id, env_description)

    @staticmethod
    def delete_env_template_data(env_template_id, path):
        """It deletes a template.

        :param env_template_id: The env_template_id to be deleted.
        :param path: The path to check.
        """
        get_description = env_temp.EnvTemplateServices.get_description
        save_description = env_temp.EnvTemplateServices.save_description

        tmp_description = get_description(env_template_id)
        if tmp_description is None:
            msg = _('Environment Template <EnvId {0}> is not found').format(
                env_template_id)
            LOG.error(msg)
            raise exc.HTTPNotFound(explanation=msg)

        utils.TraverseHelper.remove(path, tmp_description)
        save_description(tmp_description, env_template_id)
        return tmp_description

    @staticmethod
    def put_application_data(env_template_id, data, path):
        """It stores the application data inside the template description.

        :param env_template_id: The env_template_id to obtain the data
        :param data: the template description
        :param path: Id of service for which we checking status.
        :return: The template description
        """
        get_description = env_temp.EnvTemplateServices.get_description
        save_description = env_temp.EnvTemplateServices.save_description

        temp_description = get_description(env_template_id)
        if temp_description is None:
            msg = _('Environment Template <EnvId {0}> is not found').format(
                env_template_id)
            LOG.error(msg)
            raise exc.HTTPNotFound(explanation=msg)

        count = 0
        id = path[1:].split('/')[-1]
        for service in temp_description["services"]:
            if service["?"]["id"]:
                if service["?"]["id"] == id:
                    break
                count+1

        utils.TraverseHelper.update("services/{0}".format(count),
                                    data, temp_description)
        temp_description['updated'] = str(timeutils.utcnow())
        save_description(temp_description, env_template_id)
        return data



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Add action column to deployment table.

Revision ID: 003
Revises: table deployment
Create Date: 2014-07-30 16:11:33.244

"""

# revision identifiers, used by Alembic.
revision = '003'
down_revision = '002'

from alembic import op
import sqlalchemy as sa

import murano.db.migration.helpers as helpers

MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    op.rename_table('deployment', 'task')
    op.add_column(
        'task',
        sa.Column('action', sa.types.Text())
    )
    op.create_table(
        'deployment',
        sa.Column('id', sa.String(length=36), nullable=False))

    helpers.transform_table(
        'status', {'deployment_id': 'task_id'}, {},
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('entity_id', sa.String(length=255), nullable=True),
        sa.Column('entity', sa.String(length=10), nullable=True),
        sa.Column('task_id', sa.String(length=36), nullable=True),
        sa.Column('text', sa.Text(), nullable=False),
        sa.Column('level', sa.String(length=32), nullable=False),
        sa.Column('details', sa.Text(), nullable=True),
        sa.ForeignKeyConstraint(['task_id'], ['task.id'], ),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    op.drop_table('deployment')


def downgrade():
    op.drop_column('task', 'action')
    op.rename_table('task', 'deployment')

    op.create_table(
        'task',
        sa.Column('id', sa.String(length=36), nullable=False))

    helpers.transform_table(
        'status', {'task_id': 'deployment_id'}, {},
        sa.Column('created', sa.DateTime(), nullable=False),
        sa.Column('updated', sa.DateTime(), nullable=False),
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('entity_id', sa.String(length=255), nullable=True),
        sa.Column('entity', sa.String(length=10), nullable=True),
        sa.Column('deployment_id', sa.String(length=36), nullable=True),
        sa.Column('text', sa.Text(), nullable=False),
        sa.Column('level', sa.String(length=32), nullable=False),
        sa.Column('details', sa.Text(), nullable=True),
        sa.ForeignKeyConstraint(['deployment_id'], ['deployment.id'], ),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET
    )

    op.drop_table('task')
    # end Alembic commands #



# Copyright (c) 2015 Telefonica I+D.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from murano.common.i18n import _
from murano.common import uuidutils
from murano.db import models
from murano.db import session as db_session

from oslo_db import exception as db_exc
from oslo_log import log as logging
from sqlalchemy.sql import or_

LOG = logging.getLogger(__name__)


class EnvTemplateServices(object):
    @staticmethod
    def get_env_templates_by(filters):
        """Returns list of environment-templates.

           :param filters: property filters
           :return: Returns list of environment-templates
        """

        unit = db_session.get_session()
        templates = unit.query(models.EnvironmentTemplate). \
            filter_by(**filters).all()

        return templates

    @staticmethod
    def get_env_templates_or_by(filters):
        """Returns list of environment-templates.

           :param filters: property filters
           :return: Returns list of environment-templates
        """
        unit = db_session.get_session()
        templates = unit.query(models.EnvironmentTemplate). \
            filter(or_(*filters)).all()

        return templates

    @staticmethod
    def create(env_template_params, tenant_id):
        """Creates environment-template with specified params, in particular - name.

           :param env_template_params: Dict, e.g. {'name': 'temp-name'}
           :param tenant_id: Tenant Id
           :return: Created Template
        """

        env_template_params['id'] = uuidutils.generate_uuid()
        env_template_params['tenant_id'] = tenant_id
        env_template = models.EnvironmentTemplate()
        env_template.update(env_template_params)

        unit = db_session.get_session()
        with unit.begin():
            try:
                unit.add(env_template)
            except db_exc.DBDuplicateEntry:
                msg = _('Environment template specified name already exists')
                LOG.error(msg)
                raise db_exc.DBDuplicateEntry(explanation=msg)
        env_template.update({'description': env_template_params})
        env_template.save(unit)

        return env_template

    @staticmethod
    def delete(env_template_id):
        """Deletes template.

           :param env_template_id: Template that is going to be deleted
        """

        env_temp_description = EnvTemplateServices.get_description(
            env_template_id)
        env_temp_description['description'] = None
        EnvTemplateServices.save_description(
            env_temp_description, env_template_id)

    @staticmethod
    def remove(env_template_id):
        """It deletes the environment template from database.

           :param env_template_id: Template Id to be deleted.
        """

        unit = db_session.get_session()
        template = unit.query(models.EnvironmentTemplate).get(env_template_id)
        if template:
            with unit.begin():
                unit.delete(template)

    @staticmethod
    def update(env_template_id, body):
        """It updates the description of a environment template.

           :param env_template_id: Template Id to be deleted.
           :param body: The description to be updated.
           :return the template description updated
        """

        unit = db_session.get_session()
        template = unit.query(models.EnvironmentTemplate).get(env_template_id)
        template.update(body)
        template.save(unit)
        return template

    @staticmethod
    def get_description(env_template_id):
        """Returns environment template description for specified template.

           :param env_template_id: Template Id
           :return: environment-template Description Object
        """

        template = EnvTemplateServices.get_env_template(env_template_id)
        if template is None:
            raise ValueError("The environment template does not exists")
        return template.description

    @staticmethod
    def get_application_description(env_template_id):
        """Returns environment template description for specified applications.

           :param env_template_id: Template Id
           :return: Template Description Object
        """

        env_temp_desc = EnvTemplateServices.get_description(env_template_id)
        if "services" not in env_temp_desc:
            return []
        else:
            return env_temp_desc['services']

    @staticmethod
    def save_description(env_template_des, env_template_id=None):
        """Saves environment template description to specified session.

           :param env_template_des: Template Description
           :param env_template_id: The template ID.
        """
        unit = db_session.get_session()
        template = unit.query(models.EnvironmentTemplate).get(env_template_id)
        template.update({'description': env_template_des})
        template.save(unit)

    @staticmethod
    def env_template_exist(env_template_id):
        """It checks if the environment template exits in database.

           :param env_template_id: The template ID
        """

        template = EnvTemplateServices.get_env_template(env_template_id)
        if template is None:
            return False
        else:
            return True

    @staticmethod
    def get_env_template(env_template_id):
        """It obtains the environment template information from the database.

           :param env_template_id: The template ID
        """
        session = db_session.get_session()
        return session.query(models.EnvironmentTemplate).get(env_template_id)

    @staticmethod
    def clone(env_template_id, tenant_id, env_template_name, is_public):
        """Clones environment-template with specified params, in particular - name.

           :param env_template_params: Dict, e.g. {'name': 'temp-name'}
           :param tenant_id: Tenant Id
           :return: Created Template
        """

        template = EnvTemplateServices.get_env_template(env_template_id)
        env_template_params = template.to_dict()
        env_template_params['id'] = uuidutils.generate_uuid()
        env_template_params['tenant_id'] = tenant_id
        env_template_params['name'] = env_template_name
        env_template_params['is_public'] = is_public
        env_temp_desc = EnvTemplateServices.get_description(env_template_id)
        if "services" in env_temp_desc:
            env_template_params['services'] = env_temp_desc['services']

        env_template = models.EnvironmentTemplate()
        env_template.update(env_template_params)

        unit = db_session.get_session()
        with unit.begin():
            unit.add(env_template)
        env_template.update({'description': env_template_params})
        env_template.save(unit)
        return env_template



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six


class TokenSanitizer(object):
    """Helper class for cleaning some object from different passwords/tokens

    Simply searches attribute with `look a like` name as one of
    the token and replace it value with message.
    """
    def __init__(self, tokens=('token', 'pass', 'trustid'),
                 message='*** SANITIZED ***'):
        """Init method of TokenSanitizer

        :param tokens:  iterable with tokens
        :param message: string by which each token going to be replaced
        """
        self._tokens = tokens
        self._message = message

    @property
    def tokens(self):
        """Iterable with tokens that should be sanitized."""
        return self._tokens

    @property
    def message(self):
        """String by which each token going to be replaced."""
        return self._message

    def _contains_token(self, value):
        for token in self.tokens:
            if token in value.lower():
                return True
        return False

    def sanitize(self, obj):
        """Replaces each token found in object by message.

        :param obj: dict, list, tuple, object
        :return: Sanitized object
        """
        if isinstance(obj, dict):
            return dict([self.sanitize(item) for item in six.iteritems(obj)])
        elif isinstance(obj, list):
            return [self.sanitize(item) for item in obj]
        elif isinstance(obj, tuple):
            k, v = obj
            if self._contains_token(k) and isinstance(v, six.string_types):
                return k, self.message
            return k, self.sanitize(v)
        else:
            return obj



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import testtools

from tempest import config

from murano_tempest_tests.tests.api.application_catalog.artifacts import base
from murano_tempest_tests import utils

CONF = config.CONF


class TestRepositorySanity(base.BaseArtifactsTest):

    @classmethod
    def resource_setup(cls):
        if not CONF.application_catalog.glare_backend:
            msg = ("Murano is not using GLARE backend. "
                   "Skipping GLARE tests.")
            raise cls.skipException(msg)
        super(TestRepositorySanity, cls).resource_setup()

    @testtools.testcase.attr('smoke')
    def test_get_list_packages(self):
        package_list = self.artifacts_client.get_list_packages()
        self.assertIsInstance(package_list, list)

    @testtools.testcase.attr('smoke')
    def test_upload_and_delete_package(self):
        application_name = utils.generate_name('package_test_upload')
        abs_archive_path, dir_with_archive, archive_name = \
            utils.prepare_package(application_name)
        self.addCleanup(os.remove, abs_archive_path)
        package = self.artifacts_client.upload_package(
            application_name, archive_name, dir_with_archive,
            {"categories": [], "tags": [], 'is_public': False})
        package_list = self.artifacts_client.get_list_packages()
        self.assertIn(package['id'], {pkg['id'] for pkg in package_list})
        self.artifacts_client.delete_package(package['id'])
        package_list = self.artifacts_client.get_list_packages()
        self.assertNotIn(package['id'], {pkg['id'] for pkg in package_list})



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from heatclient.v1 import stacks
import mock
from oslo_config import cfg

from murano.engine.system import heat_stack
from murano.tests.unit import base

CLS_NAME = 'murano.engine.system.heat_stack.HeatStack'
CONF = cfg.CONF


class TestHeatStack(base.MuranoTestCase):
    def setUp(self):
        super(TestHeatStack, self).setUp()
        self.heat_client_mock = mock.Mock()
        self.heat_client_mock.stacks = mock.MagicMock(spec=stacks.StackManager)
        CONF.set_override('stack_tags', ['test-murano'], 'heat',
                          enforce_type=True)
        self.mock_tag = ','.join(CONF.heat.stack_tags)
        heat_stack.HeatStack._get_token_client = mock.Mock(
            return_value=self.heat_client_mock)
        heat_stack.HeatStack._get_client = mock.Mock(
            return_value=self.heat_client_mock)
        self._this = mock.MagicMock()
        self._this.owner = None

    @mock.patch(CLS_NAME + '._wait_state')
    @mock.patch(CLS_NAME + '._get_status')
    def test_push_adds_version(self, status_get, wait_st):
        """Assert that if heat_template_version is omitted, it's added."""

        status_get.return_value = 'NOT_FOUND'
        wait_st.return_value = {}
        hs = heat_stack.HeatStack(
            self._this, 'test-stack', 'Generated by TestHeatStack')
        hs._template = {'resources': {'test': 1}}
        hs._files = {}
        hs._hot_environment = ''
        hs._parameters = {}
        hs._applied = False
        hs.push()

        hs = heat_stack.HeatStack(
            self._this, 'test-stack', 'Generated by TestHeatStack')
        hs._template = {'resources': {'test': 1}}
        hs._files = {}
        hs._parameters = {}
        hs._applied = False
        hs.push()

        expected_template = {
            'heat_template_version': '2013-05-23',
            'description': 'Generated by TestHeatStack',
            'resources': {'test': 1}
        }
        self.heat_client_mock.stacks.create.assert_called_with(
            stack_name='test-stack',
            disable_rollback=True,
            parameters={},
            template=expected_template,
            files={},
            environment='',
            tags=self.mock_tag
        )
        self.assertTrue(hs._applied)

    @mock.patch(CLS_NAME + '._wait_state')
    @mock.patch(CLS_NAME + '._get_status')
    def test_description_is_optional(self, status_get, wait_st):
        """Assert that if heat_template_version is omitted, it's added."""

        status_get.return_value = 'NOT_FOUND'
        wait_st.return_value = {}
        hs = heat_stack.HeatStack(self._this, 'test-stack', None)
        hs._template = {'resources': {'test': 1}}
        hs._files = {}
        hs._hot_environment = ''
        hs._parameters = {}
        hs._applied = False
        hs.push()

        expected_template = {
            'heat_template_version': '2013-05-23',
            'resources': {'test': 1}
        }
        self.heat_client_mock.stacks.create.assert_called_with(
            stack_name='test-stack',
            disable_rollback=True,
            parameters={},
            template=expected_template,
            files={},
            environment='',
            tags=self.mock_tag
        )
        self.assertTrue(hs._applied)

    @mock.patch(CLS_NAME + '._wait_state')
    @mock.patch(CLS_NAME + '._get_status')
    def test_heat_files_are_sent(self, status_get, wait_st):
        """Assert that if heat_template_version is omitted, it's added."""

        status_get.return_value = 'NOT_FOUND'
        wait_st.return_value = {}
        hs = heat_stack.HeatStack(self._this, 'test-stack', None)
        hs._description = None
        hs._template = {'resources': {'test': 1}}
        hs._files = {"heatFile": "file"}
        hs._hot_environment = ''
        hs._parameters = {}
        hs._applied = False
        hs.push()

        expected_template = {
            'heat_template_version': '2013-05-23',
            'resources': {'test': 1}
        }
        self.heat_client_mock.stacks.create.assert_called_with(
            stack_name='test-stack',
            disable_rollback=True,
            parameters={},
            template=expected_template,
            files={"heatFile": "file"},
            environment='',
            tags=self.mock_tag
        )
        self.assertTrue(hs._applied)

    @mock.patch(CLS_NAME + '._wait_state')
    @mock.patch(CLS_NAME + '._get_status')
    def test_heat_environments_are_sent(self, status_get, wait_st):
        """Assert that if heat_template_version is omitted, it's added."""

        status_get.return_value = 'NOT_FOUND'
        wait_st.return_value = {}
        hs = heat_stack.HeatStack(self._this, 'test-stack', None)
        hs._description = None
        hs._template = {'resources': {'test': 1}}
        hs._files = {"heatFile": "file"}
        hs._hot_environment = 'environments'
        hs._parameters = {}
        hs._applied = False
        hs.push()

        expected_template = {
            'heat_template_version': '2013-05-23',
            'resources': {'test': 1}
        }
        self.heat_client_mock.stacks.create.assert_called_with(
            stack_name='test-stack',
            disable_rollback=True,
            parameters={},
            template=expected_template,
            files={"heatFile": "file"},
            environment='environments',
            tags=self.mock_tag
        )
        self.assertTrue(hs._applied)

    @mock.patch(CLS_NAME + '.current')
    def test_update_wrong_template_version(self, current):
        """Template version other than expected should cause error."""

        hs = heat_stack.HeatStack(
            self._this, 'test-stack', 'Generated by TestHeatStack')
        hs._template = {'resources': {'test': 1}}

        invalid_template = {
            'heat_template_version': 'something else'
        }

        current.return_value = {}

        e = self.assertRaises(heat_stack.HeatStackError,
                              hs.update_template,
                              invalid_template)
        err_msg = "Currently only heat_template_version 2013-05-23 "\
                  "is supported."
        self.assertEqual(err_msg, str(e))

        # Check it's ok without a version
        hs.update_template({})
        expected = {'resources': {'test': 1}}
        self.assertEqual(expected, hs._template)

        # .. or with a good version
        hs.update_template({'heat_template_version': '2013-05-23'})
        expected['heat_template_version'] = '2013-05-23'
        self.assertEqual(expected, hs._template)

    @mock.patch(CLS_NAME + '._wait_state')
    @mock.patch(CLS_NAME + '._get_status')
    def test_heat_stack_tags_are_sent(self, status_get, wait_st):
        """Assert heat_stack tags are sent

        Assert that heat_stack `tags` parameter get push & with
        value from config parameter `stack_tags`.
        """

        status_get.return_value = 'NOT_FOUND'
        wait_st.return_value = {}
        CONF.set_override('stack_tags', ['test-murano', 'murano-tag'], 'heat',
                          enforce_type=True)
        hs = heat_stack.HeatStack(self._this, 'test-stack', None)
        hs._description = None
        hs._template = {'resources': {'test': 1}}
        hs._files = {}
        hs._hot_environment = ''
        hs._parameters = {}
        hs._applied = False
        hs._tags = ','.join(CONF.heat.stack_tags)
        hs.push()

        expected_template = {
            'heat_template_version': '2013-05-23',
            'resources': {'test': 1}
        }
        self.heat_client_mock.stacks.create.assert_called_with(
            stack_name='test-stack',
            disable_rollback=True,
            parameters={},
            template=expected_template,
            files={},
            environment='',
            tags=','.join(CONF.heat.stack_tags)
        )
        self.assertTrue(hs._applied)



# Copyright (c) 2016 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import credentials_factory as common_creds
from tempest.common import dynamic_creds
from tempest import config
from tempest.lib import base

from murano_tempest_tests import clients
from murano_tempest_tests import utils

CONF = config.CONF


class BaseArtifactsTest(base.BaseTestCase):
    """Base test class for Murano Glare tests."""

    @classmethod
    def setUpClass(cls):
        super(BaseArtifactsTest, cls).setUpClass()
        cls.resource_setup()

    @classmethod
    def tearDownClass(cls):
        cls.resource_cleanup()
        super(BaseArtifactsTest, cls).tearDownClass()

    @classmethod
    def get_client_with_isolated_creds(cls, type_of_creds="admin"):

        creds = cls.get_configured_isolated_creds(type_of_creds=type_of_creds)

        os = clients.Manager(credentials=creds)
        client = os.artifacts_client

        return client

    @classmethod
    def get_configured_isolated_creds(cls, type_of_creds='admin'):
        identity_version = CONF.identity.auth_version
        if identity_version == 'v3':
            cls.admin_role = CONF.identity.admin_role
        else:
            cls.admin_role = 'admin'
        cls.dynamic_cred = dynamic_creds.DynamicCredentialProvider(
            identity_version=CONF.identity.auth_version,
            name=cls.__name__, admin_role=cls.admin_role,
            admin_creds=common_creds.get_configured_admin_credentials(
                'identity_admin'))
        if type_of_creds == 'primary':
            creds = cls.dynamic_cred.get_primary_creds()
        elif type_of_creds == 'admin':
            creds = cls.dynamic_cred.get_admin_creds()
        elif type_of_creds == 'alt':
            creds = cls.dynamic_cred.get_alt_creds()
        else:
            creds = cls.dynamic_cred.get_credentials(type_of_creds)
        cls.dynamic_cred.type_of_creds = type_of_creds

        return creds.credentials

    @classmethod
    def verify_nonempty(cls, *args):
        if not all(args):
            msg = "Missing API credentials in configuration."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        if not CONF.service_available.murano:
            skip_msg = "Murano is disabled"
            raise cls.skipException(skip_msg)
        if not hasattr(cls, "os"):
            creds = cls.get_configured_isolated_creds(type_of_creds='primary')
            cls.os = clients.Manager(credentials=creds)
        cls.artifacts_client = cls.os.artifacts_client
        cls.application_catalog_client = cls.os.application_catalog_client

    @classmethod
    def resource_cleanup(cls):
        cls.clear_isolated_creds()

    @classmethod
    def clear_isolated_creds(cls):
        if hasattr(cls, "dynamic_cred"):
            cls.dynamic_cred.clear_creds()

    @classmethod
    def upload_package(cls, application_name, version=None, require=None):
        abs_archive_path, dir_with_archive, archive_name = \
            utils.prepare_package(application_name, version=version,
                                  add_class_name=True, require=require)
        package = cls.artifacts_client.upload_package(
            application_name, archive_name, dir_with_archive,
            {"categories": [], "tags": [], 'is_public': False})
        return package, abs_archive_path

    @staticmethod
    def create_obj_model(package):
        return {
            "name": package['display_name'],
            "?": {
                "type": package['name'],
                "id": utils.generate_uuid(),
                "classVersion": package['version']
            }
        }



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

from murano.dsl import contracts
from murano.dsl import dsl_types
from murano.dsl import exceptions
from murano.dsl import helpers


class String(contracts.ContractMethod):
    name = 'string'

    def transform(self):
        if self.value is None:
            return None
        if isinstance(self.value, six.text_type):
            return self.value
        if isinstance(self.value, six.string_types) or \
                isinstance(self.value, six.integer_types):
            return six.text_type(self.value)
        if isinstance(self.value, dsl_types.MuranoObject):
            return self.value.object_id
        if isinstance(self.value, dsl_types.MuranoObjectInterface):
            return self.value.id
        raise exceptions.ContractViolationException(
            'Value {0} violates string() contract'.format(
                helpers.format_scalar(self.value)))

    def validate(self):
        if self.value is None or isinstance(self.value, six.string_types):
            return self.value
        raise exceptions.ContractViolationException()

    def generate_schema(self):
        types = 'string'
        if '_notNull' not in self.value:
            types = [types] + ['null']

        return {
            'type': types
        }


class Bool(contracts.ContractMethod):
    name = 'bool'

    def validate(self):
        if self.value is None or isinstance(self.value, bool):
            return self.value
        raise exceptions.ContractViolationException()

    def transform(self):
        if self.value is None:
            return None
        return True if self.value else False

    def generate_schema(self):
        types = 'boolean'
        if '_notNull' not in self.value:
            types = [types] + ['null']

        return {
            'type': types
        }


class Int(contracts.ContractMethod):
    name = 'int'

    def validate(self):
        if self.value is None or isinstance(
                self.value, int) and not isinstance(self.value, bool):
            return self.value
        raise exceptions.ContractViolationException()

    def transform(self):
        if self.value is None:
            return None
        try:
            return int(self.value)
        except Exception:
            raise exceptions.ContractViolationException(
                'Value {0} violates int() contract'.format(
                    helpers.format_scalar(self.value)))

    def generate_schema(self):
        types = 'integer'
        if '_notNull' not in self.value:
            types = [types] + ['null']

        return {
            'type': types
        }


class NotNull(contracts.ContractMethod):
    name = 'not_null'

    def validate(self):
        if self.value is None:
            raise exceptions.ContractViolationException(
                'null value violates notNull() contract')
        return self.value

    def transform(self):
        return self.validate()

    def generate_schema(self):
        types = self.value.get('type')
        if isinstance(types, list) and 'null' in types:
            types.remove('null')
            if len(types) == 1:
                types = types[0]
            self.value['type'] = types
        self.value['_notNull'] = True
        return self.value



# Copyright (c) 2015 Telefonica I+D.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import fixtures


class EmptyEnvironmentTemplateFixture(fixtures.Fixture):
    def setUp(self):
        super(EmptyEnvironmentTemplateFixture, self).setUp()
        self.environment_template_desc = {
            "tenant_id": "tenant_id",
            "name": "my_template",
            "id": "template_id"
        }
        self.addCleanup(delattr, self, 'environment_template_desc')


class AppEnvTemplateFixture(fixtures.Fixture):
    def setUp(self):
        super(AppEnvTemplateFixture, self).setUp()
        self.env_template_desc = \
            {
                "services": [
                    {
                        "instance": {
                            "assignFloatingIp": "true",
                            "keyname": "mykeyname",
                            "image": "cloud-fedora-v3",
                            "flavor": "m1.medium",
                            "?": {
                                "type": "io.murano.resources.LinuxInstance",
                                "id": "ef984a74-29a4-45c0-b1dc-2ab9f075732e"
                            }
                        },
                        "name": "orion",
                        "?":
                        {
                            "_26411a1861294160833743e45d0eaad9": {
                                "name": "tomcat"
                            },
                            "type": "io.murano.apps.apache.Tomcat",
                            "id": "tomcat_id"
                        },
                        "port": "8080"
                    }, {
                        "instance": "ef984a74-29a4-45c0-b1dc-2ab9f075732e",
                        "password": "XXX", "name":
                        "mysql",
                        "?": {
                            "_26411a1861294160833743e45d0eaad9": {
                                "name": "mysql"
                            },
                            "type": "io.murano.apps.database.MySQL",
                            "id": "54aaa43d-5970"
                        }
                    }
                ],
                "tenant_id": "tenant_id",
                "name": "template_name",
                'id': 'template_id'
            }
        self.addCleanup(delattr, self, 'env_template_desc')


class ApplicationsFixture(fixtures.Fixture):

    def setUp(self):
        super(ApplicationsFixture, self).setUp()
        self.applications_desc = [
            {
                "instance": {
                    "assignFloatingIp": "true",
                    "keyname": "mykeyname",
                    "image": "cloud-fedora-v3",
                    "flavor": "m1.medium",
                    "?": {
                        "type": "io.murano.resources.LinuxInstance",
                        "id": "ef984a74-29a4-45c0-b1dc-2ab9f075732e"
                    }
                },
                "name": "orion",
                "?":
                {
                    "_26411a1861294160833743e45d0eaad9": {
                        "name": "tomcat"
                    },
                    "type": "io.murano.apps.apache.Tomcat",
                    "id": "tomcat_id"
                },
                "port": "8080"
            },
            {
                "instance": "ef984a74-29a4-45c0-b1dc-2ab9f075732e",
                "password": "XXX", "name":
                "mysql",
                "?": {
                    "_26411a1861294160833743e45d0eaad9": {
                        "name": "mysql"
                    },
                    "type": "io.murano.apps.database.MySQL",
                    "id": "54aaa43d-5970"
                }
            }
        ]
        self.addCleanup(delattr, self, 'applications_desc')


class ApplicationTomcatFixture(fixtures.Fixture):

    def setUp(self):
        super(ApplicationTomcatFixture, self).setUp()
        self.application_tomcat_desc = {
            "instance": {
                "assignFloatingIp": "true",
                "keyname": "mykeyname",
                "image": "cloud-fedora-v3",
                "flavor": "m1.medium",
                "?": {
                    "type": "io.murano.resources.LinuxInstance",
                    "id": "ef984a74-29a4-45c0-b1dc-2ab9f075732e"
                }
            },
            "name": "orion",
            "?":
            {
                "_26411a1861294160833743e45d0eaad9": {
                    "name": "tomcat"
                },
                "type": "io.murano.apps.apache.Tomcat",
                "id": "tomcat_id"
            },
            "port": "8080"
        }
        self.addCleanup(delattr, self, 'application_tomcat_desc')


class ApplicationMysqlFixture(fixtures.Fixture):

    def setUp(self):
        super(ApplicationMysqlFixture, self).setUp()
        self.application_mysql_desc = {
            "instance": "ef984a74-29a4-45c0-b1dc-2ab9f075732e",
            "password": "XXX", "name":
            "mysql",
            "?": {
                "_26411a1861294160833743e45d0eaad9": {
                    "name": "mysql"
                },
                "type": "io.murano.apps.database.MySQL",
                "id": "54aaa43d-5970"
            }
        }
        self.addCleanup(delattr, self, 'application_mysql_desc')



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json
import multiprocessing
import socket
import time

import eventlet
from oslo_config import cfg
from oslo_log import log as logging
from oslo_service import service
import psutil

from murano.api import v1
from murano.api.v1 import request_statistics
from murano.common.i18n import _LE
from murano.db.services import stats as db_stats

CONF = cfg.CONF

CONF_STATS = CONF.stats
LOG = logging.getLogger(__name__)


class StatsCollectingService(service.Service):
    def __init__(self):
        super(StatsCollectingService, self).__init__()
        request_statistics.init_stats()
        self._hostname = socket.gethostname()
        self._stats_db = db_stats.Statistics()
        self._prev_time = time.time()

    def start(self):
        super(StatsCollectingService, self).start()
        self.tg.add_thread(self._collect_stats_loop)

    def stop(self):
        super(StatsCollectingService, self).stop()

    def _collect_stats_loop(self):
        period = CONF_STATS.period * 60
        while True:
            self.update_stats()
            eventlet.sleep(period)

    def update_stats(self):
        LOG.debug("Updating statistic information.")
        LOG.debug("Stats object: {stats}".format(stats=v1.stats))
        LOG.debug("Stats: (Requests: {amount}  Errors: {error}  "
                  "Ave.Res.Time {time:2.4f}\n Per tenant: {req_count})".format(
                      amount=v1.stats.request_count,
                      error=v1.stats.error_count,
                      time=v1.stats.average_time,
                      req_count=v1.stats.requests_per_tenant))
        try:
            stats = self._stats_db.get_stats_by_host(self._hostname)
            if stats is None:
                self._stats_db.create(self._hostname,
                                      v1.stats.request_count,
                                      v1.stats.error_count,
                                      v1.stats.average_time,
                                      v1.stats.requests_per_tenant,
                                      multiprocessing.cpu_count(),
                                      psutil.cpu_percent())
                return

            now = time.time()
            t_delta = now - self._prev_time
            requests_per_second = (v1.stats.request_count -
                                   stats.request_count) / t_delta
            errors_per_second = (v1.stats.error_count -
                                 stats.error_count) / t_delta
            self._prev_time = now
            stats.request_count = v1.stats.request_count
            stats.error_count = v1.stats.error_count
            stats.average_response_time = v1.stats.average_time
            stats.requests_per_tenant = json.dumps(v1.stats.
                                                   requests_per_tenant)
            stats.requests_per_second = requests_per_second
            stats.errors_per_second = errors_per_second
            stats.cpu_percent = psutil.cpu_percent()
            self._stats_db.update(self._hostname, stats)
        except Exception as e:
            LOG.exception(_LE("Failed to get statistics object from a "
                              "database. {error_code}").format(error_code=e))



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from murano_tempest_tests.tests.api.application_catalog import base
from murano_tempest_tests import utils


class TestEnvironments(base.BaseApplicationCatalogTest):

    @classmethod
    def resource_setup(cls):
        super(TestEnvironments, cls).resource_setup()
        name = utils.generate_name(cls.__name__)
        cls.environment = cls.application_catalog_client.\
            create_environment(name)

    @classmethod
    def resource_cleanup(cls):
        cls.application_catalog_client.\
            delete_environment(cls.environment['id'])
        super(TestEnvironments, cls).resource_cleanup()

    @testtools.testcase.attr('smoke')
    def test_list_environments(self):
        environments_list = self.application_catalog_client.\
            get_environments_list()
        self.assertIsInstance(environments_list, list)

    @testtools.testcase.attr('smoke')
    def test_create_and_delete_environment(self):
        environments_list = self.application_catalog_client.\
            get_environments_list()
        name = utils.generate_name('create_and_delete_env')
        environment = self.application_catalog_client.create_environment(name)
        self.assertEqual(name, environment['name'])
        upd_environments_list = self.application_catalog_client.\
            get_environments_list()
        self.assertEqual(len(environments_list) + 1,
                         len(upd_environments_list))
        self.application_catalog_client.delete_environment(environment['id'])
        upd_environments_list = self.application_catalog_client.\
            get_environments_list()
        self.assertEqual(len(environments_list),
                         len(upd_environments_list))

    @testtools.testcase.attr('smoke')
    def test_create_and_delete_environment_with_unicode_name(self):
        environments_list = self.application_catalog_client.\
            get_environments_list()
        name = u'$yaql \u2665 unicode'
        environment = self.application_catalog_client.create_environment(name)
        self.assertEqual(name, environment['name'])
        upd_environments_list = self.application_catalog_client.\
            get_environments_list()
        self.assertEqual(len(environments_list) + 1,
                         len(upd_environments_list))
        self.application_catalog_client.delete_environment(environment['id'])
        upd_environments_list = self.application_catalog_client.\
            get_environments_list()
        self.assertEqual(len(environments_list),
                         len(upd_environments_list))

    @testtools.testcase.attr('smoke')
    def test_get_environment(self):
        environment = self.application_catalog_client.\
            get_environment(self.environment['id'])
        self.assertEqual(self.environment['name'], environment['name'])

    @testtools.testcase.attr('smoke')
    def test_update_environment(self):
        environment = self.application_catalog_client.\
            update_environment(self.environment['id'])
        self.assertIsNot(self.environment['name'], environment['name'])



#!/usr/bin/env python
# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""pylint error checking."""

from __future__ import print_function

import json
import os
import re
import sys

from pylint import lint
from pylint.reporters import text
from six.moves import cStringIO as StringIO

# enabled checks
# http://pylint-messages.wikidot.com/all-codes
ENABLED_CODES = (
    # refactor category
    "R0801", "R0911", "R0912", "R0913", "R0914", "R0915",
    # warning category
    "W0612", "W0613", "W0703",
    # convention category
    "C1001")

LINE_PATTERN = r"(\S+):(\d+): \[(\S+)(, \S*)?] (.*)"

KNOWN_PYLINT_EXCEPTIONS_FILE = "tools/pylint_exceptions"


class LintOutput(object):

    _cached_filename = None
    _cached_content = None

    def __init__(self, filename, lineno, line_content, code, message,
                 lintoutput, additional_content):
        self.filename = filename
        self.lineno = lineno
        self.line_content = line_content
        self.code = code
        self.message = message
        self.lintoutput = lintoutput
        self.additional_content = additional_content

    @classmethod
    def get_duplicate_code_location(cls, remaining_lines):
        module, lineno = remaining_lines.pop(0)[2:].split(":")
        filename = module.replace(".", os.sep) + ".py"
        return filename, int(lineno)

    @classmethod
    def get_line_content(cls, filename, lineno):
        if cls._cached_filename != filename:
            with open(filename) as f:
                cls._cached_content = list(f.readlines())
                cls._cached_filename = filename
        # find first non-empty line
        lineno -= 1
        while True:
            line_content = cls._cached_content[lineno].rstrip()
            lineno += 1
            if line_content:
                return line_content

    @classmethod
    def from_line(cls, line, remaining_lines):
        m = re.search(LINE_PATTERN, line)
        if not m:
            return None
        matched = m.groups()
        filename, lineno, code, message = (matched[0], int(matched[1]),
                                           matched[2], matched[-1])
        additional_content = None

        # duplicate code output needs special handling
        if "duplicate-code" in code:
            line_count = 0
            for next_line in remaining_lines:
                if re.search(LINE_PATTERN, next_line):
                    break
                line_count += 1
            if line_count:
                additional_content = remaining_lines[0:line_count]

            filename, lineno = cls.get_duplicate_code_location(remaining_lines)
            # fixes incorrectly reported file path
            line = line.replace(matched[0], filename)
            line = line.replace(":%s:" % matched[1], "")

        line_content = cls.get_line_content(filename, lineno)
        return cls(filename, lineno, line_content, code, message,
                   line.rstrip(), additional_content)

    @classmethod
    def from_msg_to_dict(cls, msg):
        """Creates dict from pylint msg

        From the output of pylint msg, to a dict, where each key
        is a unique error identifier, value is a list of LintOutput
        """
        result = {}
        lines = msg.splitlines()
        while lines:
            line = lines.pop(0)
            obj = cls.from_line(line, lines)
            if not obj:
                continue
            key = obj.key()
            if key not in result:
                result[key] = []
            result[key].append(obj)
        return result

    def key(self):
        return self.message, self.line_content.strip()

    def json(self):
        return json.dumps(self.__dict__)

    def review_str(self):
        kargs = {"filename": self.filename,
                 "lineno": self.lineno,
                 "line_content": self.line_content,
                 "code": self.code,
                 "message": self.message}
        return ("File %(filename)s\nLine %(lineno)d:%(line_content)s\n"
                "%(code)s: %(message)s" % kargs)


class ErrorKeys(object):

    @classmethod
    def print_json(cls, errors, output=sys.stdout):
        print("# automatically generated by tools/lintstack.py", file=output)
        for i in sorted(errors.keys()):
            print(json.dumps(i), file=output)

    @classmethod
    def from_file(cls, filename):
        keys = set()
        for line in open(filename):
            if line and line[0] != "#":
                d = json.loads(line)
                keys.add(tuple(d))
        return keys


def run_pylint():
    buff = StringIO()
    reporter = text.ParseableTextReporter(output=buff)
    args = ["-rn", "--disable=all", "--enable=" + ",".join(ENABLED_CODES),
            "murano"]
    lint.Run(args, reporter=reporter, exit=False)
    val = buff.getvalue()
    buff.close()
    return val


def generate_error_keys(msg=None):
    print("Generating", KNOWN_PYLINT_EXCEPTIONS_FILE)
    if msg is None:
        msg = run_pylint()

    errors = LintOutput.from_msg_to_dict(msg)
    with open(KNOWN_PYLINT_EXCEPTIONS_FILE, "w") as f:
        ErrorKeys.print_json(errors, output=f)


def validate(newmsg=None):
    print("Loading", KNOWN_PYLINT_EXCEPTIONS_FILE)
    known = ErrorKeys.from_file(KNOWN_PYLINT_EXCEPTIONS_FILE)
    if newmsg is None:
        print("Running pylint. Be patient...")
        newmsg = run_pylint()
    errors = LintOutput.from_msg_to_dict(newmsg)

    print()
    print("Unique errors reported by pylint: was %d, now %d."
          % (len(known), len(errors)))
    passed = True
    for err_key, err_list in errors.items():
        for err in err_list:
            if err_key not in known:
                print()
                print(err.lintoutput)
                print(err.review_str())
                if err.additional_content:
                    max_len = max(map(len, err.additional_content))
                    print("-" * max_len)
                    print(os.linesep.join(err.additional_content))
                    print("-" * max_len)
                passed = False
    if passed:
        print("Congrats! pylint check passed.")
        redundant = known - set(errors.keys())
        if redundant:
            print("Extra credit: some known pylint exceptions disappeared.")
            for i in sorted(redundant):
                print(json.dumps(i))
            print("Consider regenerating the exception file if you will.")
    else:
        print()
        print("Please fix the errors above. If you believe they are false "
              "positives, run 'tools/lintstack.py generate' to overwrite.")
        sys.exit(1)


def usage():
    print("""Usage: tools/lintstack.py [generate|validate]
    To generate pylint_exceptions file: tools/lintstack.py generate
    To validate the current commit: tools/lintstack.py
    """)


def main():
    option = "validate"
    if len(sys.argv) > 1:
        option = sys.argv[1]
    if option == "generate":
        generate_error_keys()
    elif option == "validate":
        validate()
    else:
        usage()


if __name__ == "__main__":
    main()



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import yaml
import yaml.composer
import yaml.constructor

from murano.dsl import dsl_types
from murano.dsl import helpers
from murano.dsl import yaql_expression


@helpers.memoize
def get_loader(version):
    version = helpers.parse_version(version)

    class MuranoPlDict(dict):
        pass

    class YaqlExpression(yaql_expression.YaqlExpression):
        @staticmethod
        def match(expr):
            return yaql_expression.YaqlExpression.is_expression(expr, version)

    def load(contents, file_id):
        def build_position(node):
            return dsl_types.ExpressionFilePosition(
                file_id,
                node.start_mark.line + 1,
                node.start_mark.column + 1,
                node.end_mark.line + 1,
                node.end_mark.column + 1)

        class MuranoPlYamlConstructor(yaml.constructor.SafeConstructor):
            def construct_yaml_map(self, node):
                data = MuranoPlDict()
                data.source_file_position = build_position(node)
                yield data
                value = self.construct_mapping(node)
                data.update(value)

        class YaqlYamlLoader(yaml.SafeLoader, MuranoPlYamlConstructor):
            pass

        YaqlYamlLoader.add_constructor(
            u'tag:yaml.org,2002:map',
            MuranoPlYamlConstructor.construct_yaml_map)

        # workaround for PyYAML bug: http://pyyaml.org/ticket/221
        resolvers = {}
        for k, v in yaml.SafeLoader.yaml_implicit_resolvers.items():
            resolvers[k] = v[:]
        YaqlYamlLoader.yaml_implicit_resolvers = resolvers

        def yaql_constructor(loader, node):
            value = loader.construct_scalar(node)
            result = yaql_expression.YaqlExpression(value, version)
            result.source_file_position = build_position(node)
            return result

        YaqlYamlLoader.add_constructor(u'!yaql', yaql_constructor)
        YaqlYamlLoader.add_implicit_resolver(u'!yaql', YaqlExpression, None)
        return list(filter(
            lambda t: t,
            yaml.load_all(contents, Loader=YaqlYamlLoader))
        )

    return load



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log as logging

from murano.common import wsgi

ssl_middleware_opts = [
    cfg.StrOpt('secure_proxy_ssl_header',
               default='X-Forwarded-Proto',
               help="The HTTP Header that will be used to determine which "
                    "the original request protocol scheme was, even if it was "
                    "removed by an SSL terminator proxy.")
]
cfg.CONF.register_opts(ssl_middleware_opts)
LOG = logging.getLogger(__name__)


class SSLMiddleware(wsgi.Middleware):
    """Replaces the request wsgi.url_scheme environment

    A middleware that replaces the request wsgi.url_scheme environment
    variable with the value of HTTP header configured in
    secure_proxy_ssl_header if exists in the incoming request.
    This is useful if the server is behind a SSL termination proxy.
    """

    @classmethod
    def factory(cls, global_conf, **local_conf):
        def filter(app):
            return cls(app)
        return filter

    def __init__(self, application):
        super(SSLMiddleware, self).__init__(application)
        self.secure_proxy_ssl_header = 'HTTP_{0}'.format(
            cfg.CONF.secure_proxy_ssl_header.upper().replace('-', '_'))

    def process_request(self, req):
        LOG.debug('Default url_scheme: {0}. {1}: {2}'.format(
            req.environ['wsgi.url_scheme'], self.secure_proxy_ssl_header,
            req.environ.get(self.secure_proxy_ssl_header)))
        req.environ['wsgi.url_scheme'] = req.environ.get(
            self.secure_proxy_ssl_header, req.environ['wsgi.url_scheme'])



# Copyright (c) 2015 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import muranoclient.common.exceptions as murano_exceptions
from nose.plugins.attrib import attr as tag

import murano.tests.functional.common.utils as common_utils
import murano.tests.functional.integration.integration_base as core


class PolicyEnforcementTest(core.CongressIntegration):

    @classmethod
    def setUpClass(cls):
        super(PolicyEnforcementTest, cls).setUpClass()

        cls._create_policy(["murano", "murano_system"])
        cls._create_policy(["murano_action"], kind="action")

        with common_utils.ignored(murano_exceptions.HTTPInternalServerError):
            cls._upload_policy_enf_app()

    @classmethod
    def tearDownClass(cls):
        cls.purge_uploaded_packages()

    def tearDown(self):
        super(PolicyEnforcementTest, self).tearDown()
        self.purge_environments()

    @tag('all', 'coverage')
    def test_deploy_policy_fail_key(self):
        """Test expects failure due to empty key name.

        In rules_murano_system.txt file are defined congress
        rules preventing deploy environment where instances
        have empty keyname property. In other words admin
        prevented spawn instance without assigned key pair.
        """

        self._check_deploy_failure(
            self._create_test_app(key='',
                                  flavor='m1.small'),
            'missing key')

    @tag('all', 'coverage')
    def test_deploy_policy_fail_flavor(self):
        """Test expects failure due to blacklisted flavor

        In rules_murano_system.txt file are defined congress
        rules preventing deploy environment where instances
        have flavor property set to 'really.bad.flavor'.
        """

        self._check_deploy_failure(
            self._create_test_app(flavor='really.bad.flavor',
                                  key='test-key'),
            'bad flavor')

    @tag('all', 'coverage')
    def test_set_property_policy(self):
        """Tests environment modification by policy

        In rules_murano_system.txt file are defined congress
        rules changing flavor property. There are defined
        synonyms for 'really.bad.flavor'. One of such synonyms
        is 'horrible.flavor' Environment is modified prior deployment.
        The synonym name 'horrible.flavor' is set to original
        value 'really.bad.flavor' and then deployment is aborted
        because instances of 'really.bad.flavor' are prevented
        to be deployed like for the test above.
        """

        self._check_deploy_failure(
            self._create_test_app(key="test-key",
                                  flavor="horrible.flavor"),
            "bad flavor")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""A middleware that turns exceptions into parsable string. Inspired by
Cinder's faultwrapper
"""

import sys
import traceback

from oslo_config import cfg
import six
import webob

from murano.common import wsgi
from murano.packages import exceptions as pkg_exc


class HTTPExceptionDisguise(Exception):
    """Disguises HTTP exceptions

    Disguises HTTP exceptions so they can be handled by the webob fault
    application in the wsgi pipeline.
    """

    def __init__(self, exception):
        self.exc = exception
        self.tb = sys.exc_info()[2]


class Fault(object):

    def __init__(self, error):
        self.error = error

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        if req.content_type == 'application/xml':
            serializer = wsgi.XMLDictSerializer()
        else:
            serializer = wsgi.JSONDictSerializer()
        resp = webob.Response(request=req)
        default_webob_exc = webob.exc.HTTPInternalServerError()
        resp.status_code = self.error.get('code', default_webob_exc.code)
        serializer.default(resp, self.error)
        return resp


class FaultWrapper(wsgi.Middleware):
    """Replace error body with something the client can parse."""

    @classmethod
    def factory(cls, global_conf, **local_conf):
        def filter(app):
            return cls(app)
        return filter

    error_map = {
        'ValueError': webob.exc.HTTPBadRequest,
        'LookupError': webob.exc.HTTPNotFound,
        'PackageClassLoadError': webob.exc.HTTPBadRequest,
        'PackageUILoadError': webob.exc.HTTPBadRequest,
        'PackageLoadError': webob.exc.HTTPBadRequest,
        'PackageFormatError': webob.exc.HTTPBadRequest,
    }

    def _map_exception_to_error(self, class_exception):
        if class_exception == Exception:
            return webob.exc.HTTPInternalServerError

        if class_exception.__name__ not in self.error_map:
            return self._map_exception_to_error(class_exception.__base__)

        return self.error_map[class_exception.__name__]

    def _error(self, ex):

        trace = None
        webob_exc = None
        if isinstance(ex, HTTPExceptionDisguise):
            # An HTTP exception was disguised so it could make it here
            # let's remove the disguise and set the original HTTP exception
            if cfg.CONF.debug:
                trace = ''.join(traceback.format_tb(ex.tb))
            ex = ex.exc
            webob_exc = ex

        ex_type = ex.__class__.__name__

        full_message = six.text_type(ex)
        if full_message.find('\n') > -1:
            message, msg_trace = full_message.split('\n', 1)
        else:
            msg_trace = traceback.format_exc()
            message = full_message

        if isinstance(ex, pkg_exc.PackageException):
            message = ex.message

        if cfg.CONF.debug and not trace:
            trace = msg_trace

        if not webob_exc:
            webob_exc = self._map_exception_to_error(ex.__class__)

        error = {
            'code': webob_exc.code,
            'title': webob_exc.title,
            'explanation': webob_exc.explanation,
            'error': {
                'message': message,
                'type': ex_type,
                'traceback': trace,
            }
        }

        return error

    def process_request(self, req):
        try:
            return req.get_response(self.application)
        except Exception as exc:
            return req.get_response(Fault(self._error(exc)))



# Copyright 2015 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from alembic import op
import sqlalchemy as sa

"""add_locks

Revision ID: 007
Revises: 006
Create Date: 2015-04-08 14:01:06.458512

"""

# revision identifiers, used by Alembic.
revision = '007'
down_revision = '006'

MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    op.create_table(
        'locks',
        sa.Column('id', sa.String(length=50), nullable=False),
        sa.Column('ts', sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint('id'),
        mysql_engine=MYSQL_ENGINE,
        mysql_charset=MYSQL_CHARSET)


def downgrade():
    op.drop_table('locks')



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from murano.common import utils
from murano.tests.unit import base


class UtilsTests(base.MuranoTestCase):
    def test_validate_quotes(self):
        self.assertEqual(True, utils.validate_quotes('"ab"'))

    def test_validate_quotes_not_closed_quotes(self):
        self.assertRaises(ValueError, utils.validate_quotes, '"ab","b""')

    def test_validate_quotes_no_coma_before_opening_quotes(self):
        self.assertRaises(ValueError, utils.validate_quotes, '"ab""b"')

    def test_split_for_quotes(self):
        self.assertEqual(["a,b", "ac"], utils.split_for_quotes('"a,b","ac"'))

    def test_split_for_quotes_with_backslash(self):
        self.assertEqual(['a"bc', 'de', 'fg,h', r'klm\\', '"nop'],
                         utils.split_for_quotes(r'"a\"bc","de",'
                                                r'"fg,h","klm\\","\"nop"'))



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from murano import context
from murano.db import session


def dummy_context(user='test_username', tenant_id='test_tenant_id',
                  password='password', roles=None, user_id=None,
                  is_admin=False, request_id='dummy-request'):
    if roles is None:
        roles = []
    # NOTE(kzaitsev) passing non-False value by default to request_id, to
    # prevent generation during tests.
    return context.RequestContext.from_dict({
        'request_id': request_id,
        'tenant': tenant_id,
        'user': user,
        # 'roles': roles,  # Commented until policy check changes land
        'is_admin': is_admin,

    })


def save_models(*models):
    s = session.get_session()
    for m in models:
        m.save(s)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import fixtures
import mock
from oslo_utils import timeutils
import routes
from six.moves import urllib
import webob

from murano.api.v1 import request_statistics
from murano.api.v1 import router
from murano.common import policy
from murano.common import rpc
from murano.common import wsgi
from murano.tests.unit import base
from murano.tests.unit import utils

TEST_DEFAULT_LOGLEVELS = {'migrate': logging.WARN, 'sqlalchemy': logging.WARN}


def test_with_middleware(self, middleware, func, req, *args, **kwargs):
    @webob.dec.wsgify
    def _app(req):
        return func(req, *args, **kwargs)

    resp = middleware(_app).process_request(req)
    return resp


class FakeLogMixin(object):
    """Allows logs to be tested

    Allow logs to be tested (rather than just disabling
    logging. This is taken from heat
    """
    def setup_logging(self):
        # Assign default logs to self.LOG so we can still
        # assert on heat logs.
        self.LOG = self.useFixture(
            fixtures.FakeLogger(level=logging.DEBUG))
        base_list = set([nlog.split('.')[0]
                         for nlog in logging.Logger.manager.loggerDict])
        for base_name in base_list:
            if base_name in TEST_DEFAULT_LOGLEVELS:
                self.useFixture(fixtures.FakeLogger(
                    level=TEST_DEFAULT_LOGLEVELS[base_name],
                    name=base_name))
            elif base_name != 'murano':
                self.useFixture(fixtures.FakeLogger(
                    name=base_name))


class MuranoApiTestCase(base.MuranoWithDBTestCase, FakeLogMixin):
    # Set this if common.rpc is imported into other scopes so that
    # it can be mocked properly
    RPC_IMPORT = 'murano.common.rpc'

    def setUp(self):
        super(MuranoApiTestCase, self).setUp()

        self.setup_logging()

        # Mock the RPC classes
        self.mock_api_rpc = mock.Mock(rpc.ApiClient)
        self.mock_engine_rpc = mock.Mock(rpc.EngineClient)
        mock.patch(self.RPC_IMPORT + '.engine',
                   return_value=self.mock_engine_rpc).start()
        mock.patch(self.RPC_IMPORT + '.api',
                   return_value=self.mock_api_rpc).start()
        mock.patch('murano.db.services.environments.EnvironmentServices.'
                   'get_network_driver', return_value='neutron').start()

        self.addCleanup(mock.patch.stopall)

    def tearDown(self):
        super(MuranoApiTestCase, self).tearDown()
        timeutils.utcnow.override_time = None

    def _stub_uuid(self, values=None):
        class FakeUUID(object):
            def __init__(self, v):
                self.hex = v

        if values is None:
            values = []
        mock_uuid4 = mock.patch('uuid.uuid4').start()
        mock_uuid4.side_effect = [FakeUUID(v) for v in values]
        return mock_uuid4


class ControllerTest(object):
    """Common utilities for testing API Controllers."""

    DEFAULT_USER = 'test_user'
    DEFAULT_TENANT = 'test_tenant'

    def __init__(self, *args, **kwargs):
        super(ControllerTest, self).__init__(*args, **kwargs)

        # cfg.CONF.set_default('host', 'server.test')
        self.api_version = '1.0'
        self.tenant = 'test_tenant'
        self.mock_policy_check = None
        self.mapper = routes.Mapper()
        self.api = router.API(self.mapper)

        request_statistics.init_stats()

    def setUp(self):
        super(ControllerTest, self).setUp()

        self.is_admin = False

        policy.init(use_conf=False)
        real_policy_check = policy.check

        self._policy_check_expectations = []
        self._actual_policy_checks = []

        def wrap_policy_check(rule, ctxt, target=None, **kwargs):
            if target is None:
                target = {}
            self._actual_policy_checks.append((rule, target))
            return real_policy_check(rule, ctxt, target=target, **kwargs)

        mock.patch('murano.common.policy.check',
                   side_effect=wrap_policy_check).start()

        # Deny everything
        self._set_policy_rules({"default": "!"})

    def _environ(self, path):
        return {
            'SERVER_NAME': 'server.test',
            'SERVER_PORT': '8082',
            'SERVER_PROTOCOL': 'http',
            'SCRIPT_NAME': '/v1',
            'PATH_INFO': path,
            'wsgi.url_scheme': 'http',
            'QUERY_STRING': '',
            'CONTENT_TYPE': 'application/json',
        }

    def _simple_request(self, path, params=None, method='GET',
                        user=DEFAULT_USER, tenant=DEFAULT_TENANT):
        """Returns a request with a fake but valid-looking context

        Returns a request with a fake but valid-looking context
        and sets the request environment variables. If `params` is given,
        it should be a dictionary or sequence of tuples.
        """
        environ = self._environ(path)
        environ['REQUEST_METHOD'] = method

        if params:
            qs = urllib.parse.urlencode(params)
            environ['QUERY_STRING'] = qs

        req = wsgi.Request(environ)
        req.context = utils.dummy_context(user, tenant,
                                          is_admin=self.is_admin)
        self.context = req.context
        return req

    def _get(self, path, params=None, user=DEFAULT_USER,
             tenant=DEFAULT_TENANT):
        return self._simple_request(path, params=params, user=user,
                                    tenant=tenant)

    def _get_with_accept(self, path, params=None, user=DEFAULT_USER,
                         tenant=DEFAULT_TENANT,
                         accept='application/octet-stream'):
        req = self._simple_request(path, params=params, user=user,
                                   tenant=tenant)
        req.accept = accept
        return req

    def _delete(self, path, params=None, user=DEFAULT_USER,
                tenant=DEFAULT_TENANT):
        if params is None:
            params = {}
        return self._simple_request(path, params=params, method='DELETE',
                                    user=user, tenant=tenant)

    def _data_request(self, path, data, content_type='application/json',
                      method='POST', params=None,
                      user=DEFAULT_USER, tenant=DEFAULT_TENANT):
        if params is None:
            params = {}
        environ = self._environ(path)
        environ['REQUEST_METHOD'] = method

        req = wsgi.Request(environ)
        req.context = utils.dummy_context(user, tenant,
                                          is_admin=self.is_admin)
        self.context = req.context
        req.content_type = content_type
        req.body = data

        if params:
            qs = urllib.parse.urlencode(params)
            environ['QUERY_STRING'] = qs

        return req

    def _post(self, path, data, content_type='application/json', params=None,
              user=DEFAULT_USER, tenant=DEFAULT_TENANT):
        if params is None:
            params = {}
        return self._data_request(path, data, content_type, params=params,
                                  user=user, tenant=tenant)

    def _put(self, path, data, content_type='application/json', params=None,
             user=DEFAULT_USER, tenant=DEFAULT_TENANT):
        if params is None:
            params = {}
        return self._data_request(path, data, content_type, method='PUT',
                                  params=params, user=user, tenant=tenant)

    def _set_policy_rules(self, rules):
        policy.set_rules(rules, default_rule='default')

    def expect_policy_check(self, action, target=None):
        if target is None:
            target = {}
        self._policy_check_expectations.append((action, target))

    def _assert_policy_checks(self):
        self.assertEqual(self._policy_check_expectations,
                         self._actual_policy_checks)

    def tearDown(self):
        self._assert_policy_checks()
        policy.reset()
        super(ControllerTest, self).tearDown()



# Copyright (c) 2015 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json

import cfg
import glanceclient
from oslo_config import cfg as config
from oslo_log import log as logging

from murano.common import auth_utils
from murano.dsl import session_local_storage

CONF = config.CONF
LOG = logging.getLogger(__name__)


class GlanceClient(object):
    def __init__(self, this):
        self._owner = this.find_owner('io.murano.Environment')

    @property
    def _client(self):
        region = None if self._owner is None else self._owner['region']
        return self.create_glance_client(region)

    def list(self):
        images = self._client.images.list()
        while True:
            try:
                image = next(images)
                yield GlanceClient._format(image)
            except StopIteration:
                break

    def get_by_name(self, name):
        images = list(self._client.images.list(filters={"name": name}))
        if len(images) > 1:
            raise AmbiguousNameException(name)
        elif len(images) == 0:
            return None
        else:
            return GlanceClient._format(images[0])

    def get_by_id(self, imageId):
        image = self._client.images.get(imageId)
        return GlanceClient._format(image)

    @staticmethod
    def _format(image):
        res = {"id": image.id, "name": image.name}
        if hasattr(image, "murano_image_info"):
            res["meta"] = json.loads(image.murano_image_info)
        return res

    @classmethod
    def init_plugin(cls):
        cls.CONF = cfg.init_config(CONF)

    @staticmethod
    @session_local_storage.execution_session_memoize
    def create_glance_client(region):
        LOG.debug("Creating a glance client")
        params = auth_utils.get_session_client_parameters(
            service_type='image', conf=CONF, region=region)
        return glanceclient.Client(CONF.images.api_version, **params)


class AmbiguousNameException(Exception):
    def __init__(self, name):
        super(AmbiguousNameException, self).__init__("Image name '%s'"
                                                     " is ambiguous" % name)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import unittest

from murano.dsl import dsl
from murano.dsl import exceptions
from murano.dsl import murano_method
from murano.dsl import murano_object
from murano.dsl import murano_type
from murano.engine.system import garbage_collector
from murano.tests.unit import base


class TestGarbageCollector(base.MuranoTestCase):
    def setUp(self):
        super(TestGarbageCollector, self).setUp()

        self.mock_subscriber = mock.MagicMock(spec=murano_object.MuranoObject)
        self.mock_class = mock.MagicMock(spec=murano_type.MuranoClass)
        self.mock_method = mock.MagicMock(spec=murano_method.MuranoMethod)
        self.mock_method.name = "mockHandler"
        self.mock_class.methods = mock.PropertyMock(
            return_value={"mockHandler": self.mock_method})
        self.mock_subscriber.type = self.mock_class
        self.mock_subscriber.object_id = "1234"

    @mock.patch("murano.dsl.helpers.get_caller_context")
    @mock.patch("murano.dsl.helpers.get_this")
    def test_set_dd(self, this, caller_context):
        this.return_value = self.mock_subscriber
        target_0 = mock.MagicMock(spec=dsl.MuranoObjectInterface)
        target_0.object = mock.MagicMock(murano_object.MuranoObject)
        target_0.object.dependencies = {}
        garbage_collector.GarbageCollector.subscribe_destruction(
            target_0, handler="mockHandler")
        self.assertEqual([{"subscriber": "1234", "handler": "mockHandler"}],
                         target_0.object.dependencies["onDestruction"])

    @mock.patch("murano.dsl.helpers.get_caller_context")
    @mock.patch("murano.dsl.helpers.get_this")
    def test_unset_dd(self, this, caller_context):
        this.return_value = self.mock_subscriber
        target_1 = mock.MagicMock(spec=dsl.MuranoObjectInterface)
        target_1.object = mock.MagicMock(murano_object.MuranoObject)
        target_1.object.dependencies = (
            {"onDestruction": [{"subscriber": "1234",
                                "handler": "mockHandler"}]})
        garbage_collector.GarbageCollector.unsubscibe_destruction(
            target_1, handler="mockHandler")
        self.assertEqual([], target_1.object.dependencies["onDestruction"])

    @unittest.skip("WIP")
    @mock.patch("murano.dsl.helpers.get_caller_context")
    @mock.patch("murano.dsl.helpers.get_this")
    def test_set_wrong_handler(self, this, caller_context):
        this.return_value = self.mock_subscriber
        target_2 = mock.MagicMock(spec=dsl.MuranoObjectInterface)
        target_2.object = mock.MagicMock(murano_object.MuranoObject)
        target_2.object.dependencies = {}
        self.assertRaises(
            exceptions.NoMethodFound,
            garbage_collector.GarbageCollector.subscribe_destruction,
            target_2, handler="wrongHandler")



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_messaging.rpc import client
import six
from webob import exc

from murano.api.v1 import request_statistics
from murano.common import policy
from murano.common import rpc
from murano.common import wsgi


LOG = logging.getLogger(__name__)
API_NAME = 'Schemas'


class Controller(object):
    @request_statistics.stats_count(API_NAME, 'GetSchema')
    def get_schema(self, request, class_name, method_names=None):
        LOG.debug('GetSchema:GetSchema')
        target = {"class_name": class_name}
        policy.check("get_schema", request.context, target)
        class_version = request.GET.get('classVersion')
        package_name = request.GET.get('packageName')
        credentials = {
            'token': request.context.auth_token,
            'project_id': request.context.tenant
        }

        try:
            methods = (list(
                six.moves.map(six.text_type.strip, method_names.split(',')))
                if method_names else [])
            return rpc.engine().generate_schema(
                credentials, class_name, methods,
                class_version, package_name)
        except client.RemoteError as e:
            if e.exc_type in ('NoClassFound',
                              'NoPackageForClassFound',
                              'NoPackageFound'):
                raise exc.HTTPNotFound(e.value)
            raise


def create_resource():
    return wsgi.Resource(Controller())



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import routes

from murano.api.v1 import actions
from murano.api.v1 import catalog
from murano.api.v1 import deployments
from murano.api.v1 import environments
from murano.api.v1 import instance_statistics
from murano.api.v1 import request_statistics
from murano.api.v1 import schemas
from murano.api.v1 import services
from murano.api.v1 import sessions
from murano.api.v1 import static_actions
from murano.api.v1 import template_applications
from murano.api.v1 import templates
from murano.common import wsgi


class API(wsgi.Router):
    @classmethod
    def factory(cls, global_conf, **local_conf):
        return cls(routes.Mapper())

    def __init__(self, mapper):
        services_resource = services.create_resource()
        mapper.connect('/environments/{environment_id}/services',
                       controller=services_resource,
                       action='get',
                       conditions={'method': ['GET']}, path='')
        mapper.connect('/environments/{environment_id}/services/{path:.*?}',
                       controller=services_resource,
                       action='get',
                       conditions={'method': ['GET']}, path='')

        mapper.connect('/environments/{environment_id}/services',
                       controller=services_resource,
                       action='post',
                       conditions={'method': ['POST']}, path='')
        mapper.connect('/environments/{environment_id}/services/{path:.*?}',
                       controller=services_resource,
                       action='post',
                       conditions={'method': ['POST']}, path='')

        mapper.connect('/environments/{environment_id}/services',
                       controller=services_resource,
                       action='put',
                       conditions={'method': ['PUT']}, path='')
        mapper.connect('/environments/{environment_id}/services/{path:.*?}',
                       controller=services_resource,
                       action='put',
                       conditions={'method': ['PUT']}, path='')

        mapper.connect('/environments/{environment_id}/services',
                       controller=services_resource,
                       action='delete',
                       conditions={'method': ['DELETE']}, path='')
        mapper.connect('/environments/{environment_id}/services/{path:.*?}',
                       controller=services_resource,
                       action='delete',
                       conditions={'method': ['DELETE']}, path='')

        environments_resource = environments.create_resource()
        mapper.connect('/environments',
                       controller=environments_resource,
                       action='index',
                       conditions={'method': ['GET']})
        mapper.connect('/environments',
                       controller=environments_resource,
                       action='create',
                       conditions={'method': ['POST']})
        mapper.connect('/environments/{environment_id}',
                       controller=environments_resource,
                       action='update',
                       conditions={'method': ['PUT']})
        mapper.connect('/environments/{environment_id}',
                       controller=environments_resource,
                       action='show',
                       conditions={'method': ['GET']})
        mapper.connect('/environments/{environment_id}',
                       controller=environments_resource,
                       action='delete',
                       conditions={'method': ['DELETE']})
        mapper.connect('/environments/{environment_id}/lastStatus',
                       controller=environments_resource,
                       action='last',
                       conditions={'method': ['GET']})

        templates_resource = templates.create_resource()
        mapper.connect('/templates',
                       controller=templates_resource,
                       action='index',
                       conditions={'method': ['GET']})
        mapper.connect('/templates',
                       controller=templates_resource,
                       action='create',
                       conditions={'method': ['POST']})
        mapper.connect('/templates/{env_template_id}',
                       controller=templates_resource,
                       action='update',
                       conditions={'method': ['PUT']})
        mapper.connect('/templates/{env_template_id}',
                       controller=templates_resource,
                       action='show',
                       conditions={'method': ['GET']})
        mapper.connect('/templates/{env_template_id}',
                       controller=templates_resource,
                       action='delete',
                       conditions={'method': ['DELETE']})
        mapper.connect('/templates/{env_template_id}/create-environment',
                       controller=templates_resource,
                       action='create_environment',
                       conditions={'method': ['POST']})
        mapper.connect('/templates/{env_template_id}/clone',
                       controller=templates_resource,
                       action='clone',
                       conditions={'method': ['POST']})

        applications_resource = template_applications.create_resource()
        mapper.connect('/templates/{env_template_id}/services',
                       controller=applications_resource,
                       action='index',
                       conditions={'method': ['GET']}, path='')
        mapper.connect('/templates/{env_template_id}/services/{path:.*?}',
                       controller=applications_resource,
                       action='show',
                       conditions={'method': ['GET']}, path='')
        mapper.connect('/templates/{env_template_id}/services',
                       controller=applications_resource,
                       action='post',
                       conditions={'method': ['POST']}, path='')
        mapper.connect('/templates/{env_template_id}/services/{path:.*?}',
                       controller=applications_resource,
                       action='put',
                       conditions={'method': ['PUT']}, path='')
        mapper.connect('/templates/{env_template_id}/services/{path:.*?}',
                       controller=applications_resource,
                       action='delete',
                       conditions={'method': ['DELETE']}, path='')

        deployments_resource = deployments.create_resource()
        mapper.connect('/environments/{environment_id}/deployments',
                       controller=deployments_resource,
                       action='index',
                       conditions={'method': ['GET']})
        mapper.connect('/environments/{environment_id}/deployments/'
                       '{deployment_id}',
                       controller=deployments_resource,
                       action='statuses',
                       conditions={'method': ['GET']})

        sessions_resource = sessions.create_resource()
        mapper.connect('/environments/{environment_id}/configure',
                       controller=sessions_resource,
                       action='configure',
                       conditions={'method': ['POST']})
        mapper.connect('/environments/{environment_id}/sessions/{session_id}',
                       controller=sessions_resource,
                       action='show',
                       conditions={'method': ['GET']})
        mapper.connect('/environments/{environment_id}/sessions/{session_id}',
                       controller=sessions_resource,
                       action='delete',
                       conditions={'method': ['DELETE']})
        mapper.connect('/environments/{environment_id}/sessions/'
                       '{session_id}/deploy',
                       controller=sessions_resource,
                       action='deploy',
                       conditions={'method': ['POST']})

        statistics_resource = instance_statistics.create_resource()
        mapper.connect(
            '/environments/{environment_id}/instance-statistics/raw/'
            '{instance_id}',
            controller=statistics_resource,
            action='get_for_instance',
            conditions={'method': ['GET']})
        mapper.connect(
            '/environments/{environment_id}/instance-statistics/raw',
            controller=statistics_resource,
            action='get_for_environment',
            conditions={'method': ['GET']})
        mapper.connect(
            '/environments/{environment_id}/instance-statistics/aggregated',
            controller=statistics_resource,
            action='get_aggregated',
            conditions={'method': ['GET']})

        actions_resource = actions.create_resource()
        mapper.connect('/environments/{environment_id}/actions/{action_id}',
                       controller=actions_resource,
                       action='execute',
                       conditions={'method': ['POST']})
        mapper.connect('/environments/{environment_id}/actions/{task_id}',
                       controller=actions_resource,
                       action='get_result',
                       conditions={'method': ['GET']})

        static_actions_resource = static_actions.create_resource()
        mapper.connect('/actions',
                       controller=static_actions_resource,
                       action='execute',
                       conditions={'method': ['POST']})

        catalog_resource = catalog.create_resource()
        mapper.connect('/catalog/packages/categories',
                       controller=catalog_resource,
                       action='show_categories',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/packages/{package_id}',
                       controller=catalog_resource,
                       action='get',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/packages/{package_id}',
                       controller=catalog_resource,
                       action='delete',
                       conditions={'method': ['DELETE']})
        mapper.connect('/catalog/packages/{package_id}',
                       controller=catalog_resource,
                       action='update',
                       conditions={'method': ['PATCH']})
        mapper.connect('/catalog/packages',
                       controller=catalog_resource,
                       action='search',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/packages',
                       controller=catalog_resource,
                       action='upload',
                       conditions={'method': ['POST']})
        mapper.connect('/catalog/packages/{package_id}/ui',
                       controller=catalog_resource,
                       action='get_ui',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/packages/{package_id}/logo',
                       controller=catalog_resource,
                       action='get_logo',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/packages/{package_id}/supplier_logo',
                       controller=catalog_resource,
                       action='get_supplier_logo',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/packages/{package_id}/download',
                       controller=catalog_resource,
                       action='download',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/categories',
                       controller=catalog_resource,
                       action='list_categories',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/categories/{category_id}',
                       controller=catalog_resource,
                       action='get_category',
                       conditions={'method': ['GET']})
        mapper.connect('/catalog/categories',
                       controller=catalog_resource,
                       action='add_category',
                       conditions={'method': ['POST']})
        mapper.connect('/catalog/categories/{category_id}',
                       controller=catalog_resource,
                       action='delete_category',
                       conditions={'method': ['DELETE']})
        req_stats_resource = request_statistics.create_resource()
        mapper.connect('/stats',
                       controller=req_stats_resource,
                       action='get',
                       conditions={'method': ['GET']})
        schemas_resource = schemas.create_resource()
        mapper.connect('/schemas/{class_name}/{method_names}',
                       controller=schemas_resource,
                       action='get_schema',
                       conditions={'method': ['GET']})
        mapper.connect('/schemas/{class_name}',
                       controller=schemas_resource,
                       action='get_schema',
                       conditions={'method': ['GET']})

        super(API, self).__init__(mapper)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""empty message

Revision ID: 002
Revises: None
Create Date: 2014-06-23 16:34:33.698760

"""

# revision identifiers, used by Alembic.
revision = '002'
down_revision = '001'

from alembic import op
import sqlalchemy as sa


MYSQL_ENGINE = 'InnoDB'
MYSQL_CHARSET = 'utf8'


def upgrade():
    op.add_column(
        'package',
        sa.Column('supplier_logo', sa.types.LargeBinary)
    )
    op.add_column(
        'package',
        sa.Column('supplier', sa.types.Text())
    )
    # end Alembic commands #


def downgrade():
    op.drop_column('package', 'supplier')
    op.drop_column('package', 'supplier_logo')
    # end Alembic commands #



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import math

import netaddr
from netaddr.strategy import ipv4
import neutronclient.v2_0.client as nclient
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils
import retrying

from murano.common import auth_utils
from murano.common import exceptions as exc
from murano.common.i18n import _LI
from murano.dsl import dsl
from murano.dsl import helpers
from murano.dsl import session_local_storage

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


@dsl.name('io.murano.system.NetworkExplorer')
class NetworkExplorer(object):
    def __init__(self, this, region_name=None):
        session = helpers.get_execution_session()
        self._project_id = session.project_id
        self._settings = CONF.networking
        self._available_cidrs = self._generate_possible_cidrs()
        self._region = this.find_owner('io.murano.CloudRegion')
        self._region_name = region_name

    @staticmethod
    @session_local_storage.execution_session_memoize
    def _get_client(region_name):
        neutron_settings = CONF.neutron
        return nclient.Client(**auth_utils.get_session_client_parameters(
            service_type='network', region=region_name, conf=neutron_settings
        ))

    @property
    def _client(self):
        region = self._region_name or (
            None if self._region is None else self._region['name'])
        return self._get_client(region)

    # NOTE(starodubcevna): to avoid simultaneous router requests we use retry
    # decorator with random delay 1-10 seconds between attempts and maximum
    # delay time 30 seconds.
    @retrying.retry(retry_on_exception=lambda e: isinstance(e,
                    exc.RouterInfoException),
                    wait_random_min=1000, wait_random_max=10000,
                    stop_max_delay=30000)
    def get_default_router(self):
        router_name = self._settings.router_name

        routers = self._client.list_routers(
            tenant_id=self._project_id, name=router_name).get('routers')
        if len(routers) == 0:
            LOG.debug('Router {name} not found'.format(name=router_name))
            if self._settings.create_router:
                LOG.debug('Attempting to create Router {router}'.
                          format(router=router_name))
                external_network = self._settings.external_network
                kwargs = {'id': external_network} \
                    if uuidutils.is_uuid_like(external_network) \
                    else {'name': external_network}
                networks = self._client.list_networks(**kwargs).get('networks')
                ext_nets = list(filter(lambda n: n['router:external'],
                                       networks))
                if len(ext_nets) == 0:
                    raise KeyError('Router %s could not be created, '
                                   'no external network found' % router_name)
                nid = ext_nets[0]['id']

                body_data = {
                    'router': {
                        'name': router_name,
                        'external_gateway_info': {
                            'network_id': nid
                        },
                        'admin_state_up': True,
                    }
                }
                router = self._client.create_router(
                    body=body_data).get('router')
                LOG.info(_LI('Created router: {id}').format(id=router['id']))
                return router['id']
            else:
                raise KeyError('Router %s was not found' % router_name)
        else:
            if routers[0]['external_gateway_info'] is None:
                raise exc.RouterInfoException('Please set external gateway for'
                                              ' the router %s ' % router_name)
            router_id = routers[0]['id']
        return router_id

    def get_available_cidr(self, router_id, net_id):
        """Uses hash of network IDs to minimize the collisions

        Different nets will attempt to pick different cidrs out of available
        range.
        If the cidr is taken will pick another one.
        """
        taken_cidrs = self._get_cidrs_taken_by_router(router_id)
        id_hash = hash(net_id)
        num_fails = 0
        while num_fails < len(self._available_cidrs):
            cidr = self._available_cidrs[
                (id_hash + num_fails) % len(self._available_cidrs)]
            if any(self._cidrs_overlap(cidr, taken_cidr) for taken_cidr in
                   taken_cidrs):
                num_fails += 1
            else:
                return str(cidr)
        return None

    def get_default_dns(self):
        return self._settings.default_dns

    def get_external_network_id_for_router(self, router_id):
        router = self._client.show_router(router_id).get('router')
        if not router or 'external_gateway_info' not in router:
            return None
        return router['external_gateway_info'].get('network_id')

    def get_external_network_id_for_network(self, network_id):
        network = self._client.show_network(network_id).get('network')
        if network.get('router:external', False):
            return network_id

        # Get router interfaces of the network
        router_ports = self._client.list_ports(
            **{'device_owner': 'network:router_interface',
               'network_id': network_id}).get('ports')

        # For each router this network is connected to
        # check if the router has external_gateway set
        for router_port in router_ports:
            ext_net_id = self.getExternalNetworkIdForRouter(
                router_port.get('device_id'))
            if ext_net_id:
                return ext_net_id
        return None

    def _get_cidrs_taken_by_router(self, router_id):
        if not router_id:
            return []
        ports = self._client.list_ports(device_id=router_id)['ports']
        subnet_ids = []
        for port in ports:
            for fixed_ip in port['fixed_ips']:
                subnet_ids.append(fixed_ip['subnet_id'])

        all_subnets = self._client.list_subnets()['subnets']
        filtered_cidrs = [netaddr.IPNetwork(subnet['cidr']) for subnet in
                          all_subnets if subnet['id'] in subnet_ids]

        return filtered_cidrs

    @staticmethod
    def _cidrs_overlap(cidr1, cidr2):
        return (cidr1 in cidr2) or (cidr2 in cidr1)

    def _generate_possible_cidrs(self):
        bits_for_envs = int(
            math.ceil(math.log(self._settings.max_environments, 2)))
        bits_for_hosts = int(math.ceil(math.log(self._settings.max_hosts, 2)))
        width = ipv4.width
        mask_width = width - bits_for_hosts - bits_for_envs
        net = netaddr.IPNetwork(
            '{0}/{1}'.format(self._settings.env_ip_template, mask_width))
        return list(net.subnet(width - bits_for_hosts))

    def list_networks(self):
        return self._client.list_networks()['networks']

    def list_subnetworks(self):
        return self._client.list_subnets()['subnets']

    def list_ports(self):
        return self._client.list_ports()['ports']

    @session_local_storage.execution_session_memoize
    def list_neutron_extensions(self):
        return self._client.list_extensions()['extensions']



# Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from murano.dsl import dsl_exception
from murano.dsl import exceptions
from murano.tests.unit.dsl.foundation import object_model as om
from murano.tests.unit.dsl.foundation import test_case


class TestExecution(test_case.DslTestCase):
    def _load(self):
        return self.new_runner(
            om.Object('SampleClass1', stringProperty='STRING',
                      classProperty=om.Object(
                          'SampleClass2', class2Property='ANOTHER_STRING')))

    def test_load(self):
        self._load()

    def test_load_failure(self):
        self.assertRaises(
            exceptions.ContractViolationException,
            self.new_runner,
            om.Object('SampleClass1'))

    def test_trace(self):
        runner = self._load()
        self.assertEqual([], self.traces)
        runner.testTrace(123)
        self.assertEqual([123, 'STRING', 'ANOTHER_STRING'], self.traces)
        runner.testTrace(321)
        self.assertEqual([123, 'STRING', 'ANOTHER_STRING',
                          321, 'STRING', 'ANOTHER_STRING'],
                         self.traces)

    def test_exception(self):
        class CustomException(Exception):
            pass

        def raise_exception():
            raise CustomException()

        self.register_function(raise_exception, 'raiseException')
        runner = self._load()
        self.assertRaises(CustomException, runner.testException)
        runner.preserve_exception = True
        self.assertRaises(dsl_exception.MuranoPlException,
                          runner.testException)

    def test_return(self):
        self.assertEqual(3, self._load().testReturn(3))
